/*
 * Copyright (c) 2004, 2005 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2002-2005 Mellanox Technologies LTD. All rights reserved.
 * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef _CL_MEM_H_
#define _CL_MEM_H_

#include <complib/cl_qcomppool.h>
/****f* Component Library: Memory/cl_malloc
 * NAME
 *   cl_malloc
 *
 * DESCRIPTION
 *   The cl_malloc allocates a memory block of requested size for the user.
 *   for more details, see malloc
 * SYNOPSIS
 */
void* cl_malloc(const size_t size);
/*
 * PARAMETERS
 *   size
 *       [in] size of requested memory block in bytes.
 *
 * RETURN VALUE
 *   a pointer to the allocated block upon success. NULL in case of failure
 *
 * SEE ALSO
 *   cl_free
 *********/

/****f* Component Library: Memory/cl_calloc
 * NAME
 *   cl_malloc
 *
 * DESCRIPTION
 *   The cl_malloc allocates a zero filled memory block of requested size for the user.
 *   for more details, see calloc
 * SYNOPSIS
 */
void* cl_calloc(const size_t count, const size_t size);
/*
 * PARAMETERS
 *   count
 *       [in] number of blockes.
 *   size
 *       [in] size of each memory block in bytes.
 *
 * RETURN VALUE
 *   a pointer to the allocated block upon success. NULL in case of failure
 *
 * SEE ALSO
 *   cl_free
 *********/

/****f* Component Library: Memory/cl_free
 * NAME
 *   cl_malloc
 *
 * DESCRIPTION
 *   The cl_free frees a memory block created by cl_malloc/cl_calloc.
 *   for more details, see free
 * SYNOPSIS
 */
cl_status_t cl_free(void* mem_p);
/*
 * PARAMETERS
 *   mem_p
 *       [in] pointer to the beginning of the memory block to free.
 *
 *
 * RETURN VALUE
 *   none
 *
 * SEE ALSO
 *   cl_malloc, cl_calloc
 *********/


#endif
