/*
 *  Copyright (C) 2010-2018. Mellanox Technologies, Ltd. ALL RIGHTS RESERVED.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License"); you may
 *    not use this file except in compliance with the License. You may obtain
 *    a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 *    THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR
 *    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 *    LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS
 *    FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 *    See the Apache Version 2.0 License for specific language governing
 *    permissions and limitations under the License.
 *
 */

#ifndef __SPECTRUM2_H__
#define __SPECTRUM2_H__
#include <resource_manager/resource_manager.h>

/************************************************
 *  Local Defines
 ***********************************************/

/************************************************
 *  Local Macros
 ***********************************************/

/************************************************
 *  Local Type definitions
 ***********************************************/

/************************************************
 *  Defines
 ***********************************************/

#define SPECTRUM2_ACL_INGRESS_GROUPS_MAX                       95    /* Backward compatibility */
#define SPECTRUM2_ACL_ENGRESS_GROUPS_MAX                       31    /* Backward compatibility */
#define SPECTRUM2_ACL_GROUPS_SIZE_MAX                          16    /* cap max acl_group_size */
#define SPECTRUM2_ACL_GROUPS_NUM_MIN                           0
#define SPECTRUM2_ACL_GROUPS_NUM_MAX                           400   /* cap max acl groups */
#define SPECTRUM2_ACL_RULES_BLOCK_MAX                          20    /* the same exist in resource_manager.h */
#define SPECTRUM2_ACL_RULES_NUM_MIN                            0
#define SPECTRUM2_ACL_RULES_NUM_MAX                            UNLIMITED_SDK_TABLE_SIZE /* cap_max_rules - 16k *//*UNLIMITED - temp for MLNX_OS*/
#define SPECTRUM2_ACL_PBS_ENTRIES_MIN                          0
#define SPECTRUM2_ACL_PBS_ENTRIES_MAX                          0xc000
#define SPECTRUM2_ACL_PORT_RANGES_MAX                          16    /* cap max_acl_l4_port_ranges */
#define SPECTRUM2_ACL_REGIONS_MIN                              0
#define SPECTRUM2_ACL_REGIONS_MAX                              400   /* cap_max_acl_regions */
#define SPECTRUM2_ACL_INGRESS_TABLES_MAX                       512   /* Backward compatibility */
#define SPECTRUM2_ACL_EGRESS_TABLES_MAX                        128   /* Backward compatibility */
#define SPECTRUM2_ACL_TABLES_MAX                               400   /* cap_max_acl_regions */
#define SPECTRUM2_ACL_VLAN_GROUPS_MAX                          255   /* cap_max_acl_vlan_groups  */
#define SPECTRUM2_ACL_TRAP_GROUPS_MAX                          3     /* not handled */
#define SPECTRUM2_ACL_MAX_FLEX_KEYS                            6     /* new - cap_flexible_keys */
#define SPECTRUM2_ACL_MAX_ACTIONS_PER_RULE                     16    /* new - cap_max_action_per_rule, see RM_API_ACL_MAX_ACTIONS_IN_RULE */
#define SPECTRUM2_ACL_MAX_ACTIONS_PER_SET                      3     /* new - cap_actions_per_set */
#define SPECTRUM2_ACL_EXTENDED_ACTIONS_NUM_MIN                 0
#define SPECTRUM2_ACL_EXTENDED_ACTIONS_NUM_MAX                 UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ACL_PARSING_DEPTH                            96
#define SPECTRUM2_ACL_CUSTOM_BYTES_SET_MAX                     8     /* max number of sets */
#define SPECTRUM2_ACL_CUSTOM_BYTES_SET_SIZE_MAX                2     /* max set size */
#define SPECTRUM2_ACL_CUSTOM_BYTES_EXTRACTION_POINT_OFFSET_MAX 126

#define SPECTRUM2_ADJ_MC_NUM_MIN      0
#define SPECTRUM2_ADJ_MC_NUM_MAX      UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ERIF_LIST_NUM_MIN   0
#define SPECTRUM2_ERIF_LIST_NUM_MAX   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_RMID_NUM_MIN        0
#define SPECTRUM2_RMID_NUM_MAX        UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_NVE_MC_LIST_NUM_MIN 0
#define SPECTRUM2_NVE_MC_LIST_NUM_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_SMID_NUM_MIN        0
#define SPECTRUM2_SMID_NUM_MAX        UNLIMITED_SDK_TABLE_SIZE

#define SPECTRUM2_ROUTER_RIFS_MAX                      4000
#define SPECTRUM2_ROUTER_IPV6_LOOPBACK_RIFS_MAX        SPECTRUM2_ROUTER_RIFS_MAX
#define SPECTRUM2_ROUTER_RIFS_DONTCARE                 (SPECTRUM2_ROUTER_RIFS_MAX + 1)
#define SPECTRUM2_ROUTER_SUB_PORT_FID_START_INDEX      0x3c00   /*15k*/
#define SPECTRUM2_ROUTER_IPV4_UC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ROUTER_IPV4_MC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ROUTER_IPV6_UC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ROUTER_IPV6_MC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ROUTER_IPV4_MC_SYSTEM_MIN            0
#define SPECTRUM2_ROUTER_IPV4_MC_SYSTEM_MAX            UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ROUTER_IPV6_MC_SYSTEM_MIN            0
#define SPECTRUM2_ROUTER_IPV6_MC_SYSTEM_MAX            UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ROUTER_ECMP_MAX                      64
#define SPECTRUM2_ROUTER_ECMP_HASH_MAX                 127
#define SPECTRUM2_ROUTER_NEIGHS_MAX                    UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ROUTER_ADJ_MAX                       0x7D000  /*500k*/
#define SPECTRUM2_ROUTER_VRID_MAX                      1999
#define SPECTRUM2_ROUTER_COUNTERS_ID_MAX               0
#define SPECTRUM2_ROUTER_NEXT_HOP_MAX                  64
#define SPECTRUM2_ROUTER_ECMP_CONTAINER_TOT_WEIGHT_MAX 4096     /*4k*/
#define SPECTRUM2_SHSPM_TREE_MAX                       24
#define SPECTRUM2_SHSPM_ENTRY_MAX                      SPECTRUM2_KVD_SIZE
#define SPECTRUM2_MAX_ERIF_PER_RIGR                    9
#define SPECTRUM2_MC_RPF_GROUP_MAX                     512
#define SPECTRUM2_MC_RPF_GROUP_MIN                     0
#define SPECTRUM2_MC_RPF_GROUP_ENTRY_COST              2
#define SPECTRUM2_MC_RPF_GROUP_HW_TABLE_TYPE           RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM2_IB_ROUTER_LIDS_MAX        0
#define SPECTRUM2_IB_ROUTER_MC_LIDS_NUM_MAX 0
#define SPECTRUM2_IB_ROUTER_UC_LID_NUM_MAX  0
#define SPECTRUM2_IB_PKEY_TABLE_SIZE        0

#define SPECTRUM2_FCF_PORT_DENSITY_MAX 0
#define SPECTRUM2_FCF_ZONES_PAIRS_MAX  0
#define SPECTRUM2_FCF_ZONES_GROUPS_MAX 0
#define SPECTRUM2_FCF_GATEWAY_MAX      0
#define SPECTRUM2_FCF_RULES_MAX        0

#define SPECTRUM2_PORT_LCL_NUM_MAX             132
#define SPECTRUM2_PORT_EXT_NUM_MAX             128    /* Please note the current value must not be greater than the 'MAX_PHYPORT_NUM' and 'MAX_SWITCHX_PORTS' defines */
#define SPECTRUM2_PORT_LOG_NUM_MAX             4096
#define SPECTRUM2_PORT_MTU_MAX                 10000
#define SPECTRUM2_PORT_MTU_MIN                 68
#define SPECTRUM2_PORT_STORM_CONTROL_ID_MAX    4
#define SPECTRUM2_PORT_VEPA_CHANNELS_NUM_MAX   0
#define SPECTRUM2_PORT_SYSTEM_PORTS_MAX        128
#define SPECTRUM2_PORT_SYSTEM_PORT_MODULES_MAX 64
#define SPECTRUM2_IB_PORTS_MAX                 36

#define SPECTRUM2_LAG_NUM_MAX          128
#define SPECTRUM2_LAG_PORT_MEMBERS_MAX 64
#define SPECTRUM2_LAG_VECTORS_NUM_MAX  0

#define SPECTRUM2_SPAN_SESSION_ID_MAX_INTERNAL 7    /* supported by FW. Please note that value corresponds to MAX ID [0, 1, 2, ..., 7]. */
#define SPECTRUM2_SPAN_SESSION_ID_MAX_EXTERNAL 7    /* available to user. Please note HW doesn't allow to edit sessions, hence to user we expose less number of sessions. */

#define SPECTRUM2_FDB_MID_MIN                  0
#define SPECTRUM2_FDB_MID_MAX                  7000
#define SPECTRUM2_FDB_MID_START_INDEX          0
#define SPECTRUM2_FDB_PGT_MAX                  0
#define SPECTRUM2_FDB_UC_ADDRESS_MAX           (SPECTRUM2_KVD_SIZE * SPECTRUM2_KVD_GUARANTEED_CAPACITY)
#define SPECTRUM2_FDB_UC_ADDRESS_SDK_TABLE_MIN 0
#define SPECTRUM2_FDB_UC_ADDRESS_SDK_TABLE_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_FDB_MC_ADDRESS_MAX           (SPECTRUM2_KVD_SIZE * SPECTRUM2_KVD_GUARANTEED_CAPACITY)
#define SPECTRUM2_FDB_MC_ADDRESS_SDK_TABLE_MIN 0
#define SPECTRUM2_FDB_MC_ADDRESS_SDK_TABLE_MAX UNLIMITED_SDK_TABLE_SIZE

#define SPECTRUM2_FLOODING_TABLE_MAX         2
#define SPECTRUM2_FLOODING_TABLE_PER_VID_MAX 1

#define SPECTRUM2_SWID_ID_MAX 0

#define SPECTRUM2_POLICER_HOST_IFC_POOL_SIZE 64
#define SPECTRUM2_POLICER_POOL_SIZE          2000 - SPECTRUM2_POLICER_HOST_IFC_POOL_SIZE

#define SPECTRUM2_POLICER_BS_MIN_VALUE_PACKETS    3
#define SPECTRUM2_POLICER_BS_MIN_VALUE_BYTES      4
#define SPECTRUM2_POLICER_BS_MAX_VALUE            30
#define SPECTRUM2_POLICER_STORM_CONTROL_POOL_SIZE 640

#define SPECTRUM2_COS_TRAFFIC_CLASS_MAX             15
#define SPECTRUM2_COS_PORT_ETS_SUB_GROUP_MAX        7
#define SPECTRUM2_COS_PORT_ETS_GROUP_MAX            7
#define SPECTRUM2_COS_PORT_PRIO_MAX                 14
#define SPECTRUM2_COS_PORT_COLOR_MAX                2
#define SPECTRUM2_COS_PORT_ETS_ELEMENTS_NUM         35
#define SPECTRUM2_COS_BUFFER_64K_SIZE_MAX           63.703125
#define SPECTRUM2_COS_BUFFER_128K_SIZE_MAX          127.828125
#define SPECTRUM2_COS_REDECN_DEFAULT_AQS_WEIGHT     5
#define SPECTRUM2_COS_REDECN_RED_DROP_ENABLED       TRUE
#define SPECTRUM2_COS_REDECN_SCD                    FALSE
#define SPECTRUM2_COS_REDECN_PROFILES_MAX           3072
#define SPECTRUM2_COS_REDECN_CELL_MULTIPLIER        64
#define SPECTRUM2_COS_REDECN_DEFAULT_TC_RED_ENABLED FALSE
#define SPECTRUM2_COS_REDECN_DEFAULT_TC_ECN_ENABLED FALSE
#define SPECTRUM2_BRIDGE_NUM_MAX                    6144 /* 6k */
#define SPECTRUM2_TOTAL_TCAM                        0x4000
#define SPECTRUM2_TOTAL_L3_REPLICATION_MATRIX       0

#define SPECTRUM2_NUM_HW_CPU_INGRESS_TCLASS_MAX 32
#define SPECTRUM2_NUM_HW_DR_PATHS_MAX           0
#define SPECTRUM2_NUM_HW_TRAP_GROUPS_MAX        32
#define SPECTRUM2_TRAP_GROUP_PRIORITY_MIN       0
#define SPECTRUM2_TRAP_GROUP_PRIORITY_MAX       5
#define SPECTRUM2_NUM_RDQS_MAX                  32
#define SPECTRUM2_NUM_CPU_TCLASS                8

#define SPECTRUM2_DECAP_RULES_NUM_MIN 0
#define SPECTRUM2_DECAP_RULES_NUM_MAX UNLIMITED_SDK_TABLE_SIZE

#define SPECTRUM2_IGMP_V3_NUM_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_IGMP_V3_NUM_MIN 0

/*Telemetry */
#define SPECTRUM2_TELE_HISTOGRAM_NUM_MAX                    64
#define SPECTRUM2_TELE_HISTOGRAM_QUEUE_DEPTH_BINS           10
#define SPECTRUM2_TELE_HISTOGRAM_SAMPLE_TIME_RESOLUTION_MAX 11
#define SPECTRUM2_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_MIN    3
#define SPECTRUM2_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_MAX    28
#define SPECTRUM2_TELE_HISTOGRAM_DATA_BIN_BITS_MAX          23
#define SPECTRUM2_TELE_HISTOGRAM_MIN_BOUNDARY_MIN           1
#define SPECTRUM2_TELE_THRESHOLD_THR_MAX                    0xFFFFFF

#define SPECTRUM2_ILM_NUM_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ILM_NUM_MIN UNLIMITED_SDK_TABLE_SIZE

/*SW TABLE -> HW TABLE MAPPING */

#define SPECTRUM2_FDB_UC_MAC_HW_TABLE_TYPE /*sfd*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_FDB_MC_MAC_HW_TABLE_TYPE /*sfd*/ RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM2_ROUTER_IPV4_UC_HW_TABLE_TYPE       /*ralue*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_ROUTER_IPV6_UC_HW_TABLE_TYPE       /*ralue*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_ROUTER_IPV4_MC_HW_TABLE_TYPE       /*rmft*/ RM_HW_TABLE_TYPE_TCAM_E
#define SPECTRUM2_ROUTER_IPV6_MC_HW_TABLE_TYPE       /*rmft*/ RM_HW_TABLE_TYPE_TCAM_E
#define SPECTRUM2_ROUTER_IPV4_MC_SYSTEM_HW_TABLE_TYPE/*rmft*/ (SPECTRUM2_ROUTER_IPV4_MC_HW_TABLE_TYPE)
#define SPECTRUM2_ROUTER_IPV6_MC_SYSTEM_HW_TABLE_TYPE/*rmft*/ (SPECTRUM2_ROUTER_IPV6_MC_HW_TABLE_TYPE)

#define SPECTRUM2_ROUTER_ARP_IPV4_HW_TABLE_TYPE/*rauht*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_ROUTER_ARP_IPV6_HW_TABLE_TYPE/*rauht*/ RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM2_ROUTER_MC_REPLICATIONS_HW_TABLE_TYPE/*rigr*/ RM_HW_TABLE_TYPE_REPLICATION_MATRIX_E

#define SPECTRUM2_ROUTER_ADJACENCY_HW_TABLE_TYPE/*ratr*/ RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM2_L2_MC_MIDS_HW_TABLE_TYPE/*smid*/ RM_HW_TABLE_TYPE_PGT_E

#define SPECTRUM2_LAG_VECTORS_HW_TABLE_TYPE/*sldr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM2_L2_MC_FLOODING_HW_TABLE_TYPE/*sftr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM2_FCF_FORWORDING_HW_TABLE_TYPE/*fftr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM2_ACL_RULES_HW_TABLE_TYPE /*ptce*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM2_ACL_EXTENDED_ACTIONS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM2_ACL_PBS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM2_ACL_REGIONS_ENTRY_COST    1
#define SPECTRUM2_ACL_REGIONS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_ACL_REGIONS_TABLE_E

#define SPECTRUM2_ACL_RULES_TWO_KEY_BLOCK_NUM_MIN       0
#define SPECTRUM2_ACL_RULES_TWO_KEY_BLOCK_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ACL_RULES_TWO_KEY_BLOCK_HW_ENTRY_COST 1
#define SPECTRUM2_ACL_RULES_TWO_KEY_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM2_ACL_RULES_FOUR_KEY_BLOCK_NUM_MIN       0
#define SPECTRUM2_ACL_RULES_FOUR_KEY_BLOCK_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ACL_RULES_FOUR_KEY_BLOCK_HW_ENTRY_COST 2
#define SPECTRUM2_ACL_RULES_FOUR_KEY_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E


#define SPECTRUM2_ACL_RULES_SIX_KEY_BLOCK_NUM_MIN       0
#define SPECTRUM2_ACL_RULES_SIX_KEY_BLOCK_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_ACL_RULES_SIX_KEY_BLOCK_HW_ENTRY_COST 3
#define SPECTRUM2_ACL_RULES_SIX_KEY_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM2_ACL_RULES_SDK_TABLE_TYPE RM_SDK_TABLE_TYPE_ACL_RULES_E

#define SPECTRUM2_ACL_GROUPS_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_ACL_GROUP_TABLE_E
#define SPECTRUM2_ADJ_MC_HW_TABLE_TYPE      RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_ERIF_LIST_HW_TABLE_TYPE   RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_RMID_HW_TABLE_TYPE        RM_HW_TABLE_TYPE_PGT_E
#define SPECTRUM2_NVE_MC_LIST_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_SMID_HW_TABLE_TYPE        RM_HW_TABLE_TYPE_PGT_E

#define SPECTRUM2_TUNNEL_RTDP_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_DECAP_RULES_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM2_IGMP_V3_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM2_ILM_HW_TABLE_TYPE /*mpilm*/ RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM2_IPV6_MGR_RIPS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/*phy_entry_cost per table */

#define SPECTRUM2_FDB_UC_MAC_HW_ENTRY_COST 1
#define SPECTRUM2_FDB_MC_MAC_HW_ENTRY_COST 1

#define SPECTRUM2_ROUTER_IPV4_UC_HW_ENTRY_COST       /*ralue v4*/ 1
#define SPECTRUM2_ROUTER_IPV6_UC_HW_ENTRY_COST       /*ralue v6 (depends on prefix length)*/ 1
#define SPECTRUM2_ROUTER_IPV4_MC_HW_ENTRY_COST       /*rmft*/ 1
#define SPECTRUM2_ROUTER_IPV6_MC_HW_ENTRY_COST       /*rmftv2*/ 2
#define SPECTRUM2_ROUTER_IPV4_MC_SYSTEM_HW_ENTRY_COST/*rmft*/ (SPECTRUM2_ROUTER_IPV4_MC_HW_ENTRY_COST)
#define SPECTRUM2_ROUTER_IPV6_MC_SYSTEM_HW_ENTRY_COST/*rmftv2*/ (SPECTRUM2_ROUTER_IPV6_MC_HW_ENTRY_COST)

#define SPECTRUM2_ROUTER_ARP_IPV4_HW_ENTRY_COST/*rauht v4*/ 1
#define SPECTRUM2_ROUTER_ARP_IPV6_HW_ENTRY_COST/*rauht v6*/ 2

#define SPECTRUM2_ROUTER_MC_REPLICATIONS_HW_ENTRY_COST/*rigr*/ 1

#define SPECTRUM2_ROUTER_ADJACENCY_HW_ENTRY_COST/*ratr*/ 1

#define SPECTRUM2_L2_MC_MIDS_HW_ENTRY_COST/*smid*/ 1

#define SPECTRUM2_LAG_VECTORS_ENTRY_COST 1

#define SPECTRUM2_L2_MC_FLOODING_HW_ENTRY_COST/*sftr*/ 1

#define SPECTRUM2_FCF_FORWORDING_HW_ENTRY_COST/*fftr*/ 1

#define SPECTRUM2_ACL_RULES_HW_ENTRY_COST /*ptce*/ 3

#define SPECTRUM2_ACL_EXTENDED_ACTIONS_HW_ENTRY_COST 1

#define SPECTRUM2_ACL_PBS_HW_ENTRY_COST 1

#define SPECTRUM2_ACL_GROUPS_HW_ENTRY_COST 1

#define SPECTRUM2_ADJ_MC_HW_ENTRY_COST 1

#define SPECTRUM2_ERIF_LIST_HW_ENTRY_COST 1

#define SPECTRUM2_NVE_MC_LIST_HW_ENTRY_COST 1

#define SPECTRUM2_TUNNEL_RTDP_HW_ENTRY_COST 1

#define SPECTRUM2_DECAP_RULES_HW_ENTRY_COST 2

#define SPECTRUM2_RMID_HW_ENTRY_COST 1

#define SPECTRUM2_SMID_HW_ENTRY_COST          1
#define SPECTRUM2_IGMP_V3_HW_ENTRY_COST       1
#define SPECTRUM2_ILM_HW_ENTRY_COST           /*mpilm*/ 1
#define SPECTRUM2_IPV6_MGR_RIPS_HW_ENTRY_COST 1

#define SPECTRUM2_HEADROOM_NUM_BUFF_PER_PORT 9
/* Shared buffers - Condor */
#define SPECTRUM2_BUFFERS_TOTAL_MEMORY            (42 * 1024 * 1024)
#define SPECTRUM2_DESCRIPTOR_BUFFERS_TOTAL_MEMORY (80 * 1024)
#define SPECTRUM2_MAX_TOTAL_HEADROOM_SIZE         (1280 * 1024)

#define SPECTRUM2_SHARED_BUFF_NUM_ING_POOL       10
#define SPECTRUM2_SHARED_BUFF_NUM_EGR_POOL       11
#define SPECTRUM2_SHARED_BUFF_NUM_MC_POOL        1
#define SPECTRUM2_SHARED_BUFF_NUM_ING_DESC_POOL  9
#define SPECTRUM2_SHARED_BUFF_NUM_EGR_DESC_POOL  9
#define SPECTRUM2_SHARED_BUFF_NUM_MANAGMENT_POOL 1
#define SPECTRUM2_SHARED_BUFF_TOTAL_NUM_POOL   \
    (SPECTRUM2_SHARED_BUFF_NUM_ING_POOL +      \
     SPECTRUM2_SHARED_BUFF_NUM_EGR_POOL +      \
     SPECTRUM2_SHARED_BUFF_NUM_ING_DESC_POOL + \
     SPECTRUM2_SHARED_BUFF_NUM_EGR_DESC_POOL + \
     SPECTRUM2_SHARED_BUFF_NUM_MC_POOL +       \
     SPECTRUM2_SHARED_BUFF_NUM_MANAGMENT_POOL)
#define SPECTRUM2_SHARED_BUFF_NUM_EGR_BUFF_PER_PORT  17
#define SPECTRUM2_SHARED_BUFF_NUM_ING_BUFF_PER_PORT  9
#define SPECTRUM2_SHARED_BUFF_MAX_POOL_SIZE          (42 * 1024 * 1024)
#define SPECTRUM2_SHARED_BUFF_PORT_NUM_MAX           64
#define SPECTRUM2_SHARED_BUFF_TCLASS_NUM_MAX         64
#define SPECTRUM2_SHARED_BUFF_RET_STATISTICS_NUM_MAX 120
#define SPECTRUM2_BUFF_UNIT_SIZE                     144
#define SPECTRUM2_SHARED_BUFF_MC_MAX_NUM_PRIO        15

#define SPECTRUM2_KVD_SIZE                      0x7D000     /* 500K */
#define SPECTRUM2_KVD_HASH_SINGLE_MIN_SIZE      0
#define SPECTRUM2_KVD_HASH_DOUBLE_MIN_SIZE      0
#define SPECTRUM2_KVD_GUARANTEED_CAPACITY       0.9     /* 90% */
#define SPECTRUM2_KVD_LINEAR_VIRTUAL_TABLE_SIZE 0x800000    /* 8M */

#define SPECTRUM2_PGT_SIZE  0x7000                      /* 28K */
#define SPECTRUM2_RMPE_SIZE 0x3C00                       /* 15K */

/* SW Counter Allocation - Only used on some chips */
#define SPECTRUM2_CNTR_BANK_PAIRS     4
#define SPECTRUM2_CNTR_LINES_PER_BANK 10000
/* Flow counters */
#define SPECTRUM2_CNTR_LINES_FLOW_PKT  2
#define SPECTRUM2_CNTR_TYPE_FLOW_PKT   0x03
#define SPECTRUM2_CNTR_LINES_FLOW_BYTE 2
#define SPECTRUM2_CNTR_TYPE_FLOW_BYTE  0x03
#define SPECTRUM2_CNTR_LINES_FLOW_BOTH 2
#define SPECTRUM2_CNTR_TYPE_FLOW_BOTH  0x03
/* Router Interface counters */
#define SPECTRUM2_CNTR_LINES_RIF_BASIC    10
#define SPECTRUM2_CNTR_TYPE_RIF_BASIC     0x09
#define SPECTRUM2_CNTR_LINES_RIF_ENHANCED 30       /*28*/
#define SPECTRUM2_CNTR_TYPE_RIF_ENHANCED  0x0A
#define SPECTRUM2_CNTR_LINES_RIF_MIXED_1  20       /*18*/
#define SPECTRUM2_CNTR_TYPE_RIF_MIXED_1   0x0B
#define SPECTRUM2_CNTR_LINES_RIF_MIXED_2  20       /*18*/
#define SPECTRUM2_CNTR_TYPE_RIF_MIXED_2   0x0C

#define SPECTRUM2_FLOW_COUNTER_NUM_MIN       0
#define SPECTRUM2_FLOW_COUNTER_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_FLOW_COUNTER_HW_ENTRY_COST SPECTRUM_CNTR_LINES_FLOW_BOTH
#define SPECTRUM2_FLOW_COUNTER_HW_TABLE_TYPE RM_HW_TABLE_TYPE_FLOW_COUNTER_E

#define SPECTRUM2_RIF_COUNTER_NUM_MIN       0
#define SPECTRUM2_RIF_COUNTER_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_RIF_COUNTER_HW_ENTRY_COST 1
#define SPECTRUM2_RIF_COUNTER_HW_TABLE_TYPE RM_HW_TABLE_TYPE_FLOW_COUNTER_E

#define SPECTRUM2_RIF_COUNTER_BASIC_NUM_MIN       0
#define SPECTRUM2_RIF_COUNTER_BASIC_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_RIF_COUNTER_BASIC_HW_ENTRY_COST SPECTRUM2_CNTR_LINES_RIF_BASIC
#define SPECTRUM2_RIF_COUNTER_BASIC_HW_TABLE_TYPE RM_HW_TABLE_TYPE_FLOW_COUNTER_E

#define SPECTRUM2_RIF_COUNTER_ENHANCED_NUM_MIN       0
#define SPECTRUM2_RIF_COUNTER_ENHANCED_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_RIF_COUNTER_ENHANCED_HW_ENTRY_COST SPECTRUM2_CNTR_LINES_RIF_ENHANCED
#define SPECTRUM2_RIF_COUNTER_ENHANCED_HW_TABLE_TYPE RM_HW_TABLE_TYPE_FLOW_COUNTER_E

#define SPECTRUM2_RIF_COUNTER_SDK_TABLE_TYPE RM_SDK_TABLE_TYPE_RIF_COUNTER_E

/* Tunnel */
#define SPECTRUM2_NUM_MAX_TUNNEL_IPINIP              SPECTRUM2_ROUTER_RIFS_MAX
#define SPECTRUM2_NUM_MAX_TUNNEL_IPINIPV6            SPECTRUM2_ROUTER_IPV6_LOOPBACK_RIFS_MAX
#define SPECTRUM2_NUM_MAX_TUNNEL_NVE                 1
#define SPECTRUM2_TUNNEL_NVE_GS_FLOOD_MAX            1
#define SPECTRUM2_TUNNEL_NVE_GS_MC_MAX               1
#define SPECTRUM2_TUNNEL_RTDP_NUM_MIN                0
#define SPECTRUM2_TUNNEL_RTDP_NUM_MAX                (SPECTRUM2_NUM_MAX_TUNNEL_IPINIP + SPECTRUM2_NUM_MAX_TUNNEL_NVE)
#define SPECTRUM2_TUNNEL_IPV6_FDB_MIN                0
#define SPECTRUM2_TUNNEL_IPV6_FDB_MAX                UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_TUNNEL_IPV6_FDB_ENTRY_COST         1
#define SPECTRUM2_TUNNEL_IPV6_FDB_HW_TABLE_TYPE      RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_TUNNEL_NVE_IPV6_LIST_MIN           0
#define SPECTRUM2_TUNNEL_NVE_IPV6_LIST_MAX           UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_TUNNEL_NVE_IPV6_LIST_ENTRY_COST    1
#define SPECTRUM2_TUNNEL_NVE_IPV6_LIST_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_TUNNEL_TNUMT_IPV6_RECORDS_MAX      4
#define SPECTRUM2_IPV6_MGR_RIPS_NUM_MIN              0
#define SPECTRUM2_IPV6_MGR_RIPS_NUM_MAX              0x3FFFF /* 256k */

/* IGMP V3 */
#define SPECTRUM2_IGMP_V3_NUM_MAX_ENTRIES 10000

/* Algo-TCAM
 * TODO: Add correct Spectrum2 values */
#define SPECTRUM2_A_TCAM_ONE_KEY_BLOCK_MIN             0
#define SPECTRUM2_A_TCAM_ONE_KEY_BLOCK_MAX             0
#define SPECTRUM2_A_TCAM_ONE_KEY_BLOCK_ENTRY_COST      0
#define SPECTRUM2_A_TCAM_ONE_KEY_BLOCK_HW_TABLE_TYPE   RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM2_A_TCAM_TWO_KEYS_BLOCK_MIN            0
#define SPECTRUM2_A_TCAM_TWO_KEYS_BLOCK_MAX            0
#define SPECTRUM2_A_TCAM_TWO_KEYS_BLOCK_ENTRY_COST     0
#define SPECTRUM2_A_TCAM_TWO_KEYS_BLOCK_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM2_A_TCAM_FOUR_KEYS_BLOCK_MIN           0
#define SPECTRUM2_A_TCAM_FOUR_KEYS_BLOCK_MAX           0
#define SPECTRUM2_A_TCAM_FOUR_KEYS_BLOCK_ENTRY_COST    0
#define SPECTRUM2_A_TCAM_FOUR_KEYS_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM2_A_TCAM_SIX_KEYS_BLOCK_MIN            0
#define SPECTRUM2_A_TCAM_SIX_KEYS_BLOCK_MAX            0
#define SPECTRUM2_A_TCAM_SIX_KEYS_BLOCK_ENTRY_COST     0
#define SPECTRUM2_A_TCAM_SIX_KEYS_BLOCK_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_INVALID

/* VLAN: VID to FID */
#define SPECTRUM2_VLANS_VID_TO_FID_MIN           0
#define SPECTRUM2_VLANS_VID_TO_FID_MAX           UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_VLANS_VID_TO_FID_ENTRY_COST    1
#define SPECTRUM2_VLANS_VID_TO_FID_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/* VPORT: RQ/VID to FID */
#define SPECTRUM2_VPORTS_RQ_VID_TO_FID_MIN           0
#define SPECTRUM2_VPORTS_RQ_VID_TO_FID_MAX           UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_VPORTS_RQ_VID_TO_FID_ENTRY_COST    1
#define SPECTRUM2_VPORTS_RQ_VID_TO_FID_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/* FID */
#define SPECTRUM2_FIDS_MIN           0
#define SPECTRUM2_FIDS_MAX           0x4000 /* 16K */
#define SPECTRUM2_FIDS_ENTRY_COST    1
#define SPECTRUM2_FIDS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/* VNI */
#define SPECTRUM2_VNI_TO_FID_MIN           0
#define SPECTRUM2_VNI_TO_FID_MAX           0xFFFFFE
#define SPECTRUM2_VNI_TO_FID_ENTRY_COST    1
#define SPECTRUM2_VNI_TO_FID_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/* MPLS */
#define SPECTRUM2_MPLS_NHLFE_MIN            0
#define SPECTRUM2_MPLS_NHLFE_MAX            UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM2_MPLS_NHLFE_ENTRY_COST     1
#define SPECTRUM2_MPLS_NHLFE_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM2_PB_MPLS_ILM_MIN           0
#define SPECTRUM2_PB_MPLS_ILM_MAX           0xc000
#define SPECTRUM2_PB_MPLS_ILM_ENTRY_COST    1
#define SPECTRUM2_PB_MPLS_ILM_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM2_INVALID_SW_TABLE_TYPE RM_SDK_TABLE_TYPE_INVALID_E


/************************************************
 *  Macros
 ***********************************************/

/************************************************
 *  Type definitions
 ***********************************************/

/************************************************
 *  Global variables
 ***********************************************/
static const rm_resources_t rm_resource_spectrum2 = {
/*13*/
    SPECTRUM2_ACL_INGRESS_GROUPS_MAX,
    SPECTRUM2_ACL_ENGRESS_GROUPS_MAX,
    SPECTRUM2_ACL_GROUPS_SIZE_MAX,
    SPECTRUM2_ACL_GROUPS_NUM_MAX,
    SPECTRUM2_ACL_RULES_BLOCK_MAX,
    SPECTRUM2_ACL_PBS_ENTRIES_MAX,
    SPECTRUM2_ACL_PORT_RANGES_MAX,
    SPECTRUM2_ACL_REGIONS_MAX,
    SPECTRUM2_ACL_INGRESS_TABLES_MAX,
    SPECTRUM2_ACL_EGRESS_TABLES_MAX,
    SPECTRUM2_ACL_TABLES_MAX,
    SPECTRUM2_ACL_VLAN_GROUPS_MAX,
    SPECTRUM2_ACL_TRAP_GROUPS_MAX,
    SPECTRUM2_ACL_MAX_FLEX_KEYS,
    SPECTRUM2_ACL_MAX_ACTIONS_PER_RULE,
    SPECTRUM2_ACL_MAX_ACTIONS_PER_SET,
    SPECTRUM2_ACL_PARSING_DEPTH,
    SPECTRUM2_ACL_CUSTOM_BYTES_SET_MAX,
    SPECTRUM2_ACL_CUSTOM_BYTES_SET_SIZE_MAX,
    SPECTRUM2_ACL_CUSTOM_BYTES_EXTRACTION_POINT_OFFSET_MAX,
    SPECTRUM2_PB_MPLS_ILM_MAX,

    /*20*/
    SPECTRUM2_ROUTER_RIFS_MAX,
    SPECTRUM2_ROUTER_RIFS_DONTCARE,
    SPECTRUM2_ROUTER_SUB_PORT_FID_START_INDEX,
    SPECTRUM2_ROUTER_IPV4_UC_MAX,
    SPECTRUM2_ROUTER_IPV4_MC_MAX,
    SPECTRUM2_ROUTER_IPV6_UC_MAX,
    SPECTRUM2_ROUTER_IPV6_MC_MAX,
    SPECTRUM2_ROUTER_ECMP_MAX,
    SPECTRUM2_ROUTER_ECMP_HASH_MAX,
    SPECTRUM2_ROUTER_ADJ_MAX,
    SPECTRUM2_ROUTER_NEIGHS_MAX,
    SPECTRUM2_ROUTER_VRID_MAX,
    SPECTRUM2_ROUTER_COUNTERS_ID_MAX,
    SPECTRUM2_ROUTER_NEXT_HOP_MAX,
    SPECTRUM2_ROUTER_ECMP_CONTAINER_TOT_WEIGHT_MAX,
    SPECTRUM2_SHSPM_TREE_MAX,
    SPECTRUM2_SHSPM_ENTRY_MAX,
    SPECTRUM2_MC_RPF_GROUP_MAX,

    /*5*/
    SPECTRUM2_FCF_PORT_DENSITY_MAX,
    SPECTRUM2_FCF_ZONES_PAIRS_MAX,
    SPECTRUM2_FCF_ZONES_GROUPS_MAX,
    SPECTRUM2_FCF_GATEWAY_MAX,
    SPECTRUM2_FCF_RULES_MAX,

    /*8*/
    SPECTRUM2_PORT_LCL_NUM_MAX,
    SPECTRUM2_PORT_EXT_NUM_MAX,
    SPECTRUM2_PORT_LOG_NUM_MAX,
    SPECTRUM2_PORT_MTU_MAX,
    SPECTRUM2_PORT_MTU_MIN,
    SPECTRUM2_PORT_STORM_CONTROL_ID_MAX,
    SPECTRUM2_PORT_VEPA_CHANNELS_NUM_MAX,
    SPECTRUM2_PORT_SYSTEM_PORTS_MAX,
    SPECTRUM2_PORT_SYSTEM_PORT_MODULES_MAX,

    /*6*/
    SPECTRUM2_LAG_NUM_MAX,
    SPECTRUM2_LAG_PORT_MEMBERS_MAX,

    SPECTRUM2_SPAN_SESSION_ID_MAX_INTERNAL,
    SPECTRUM2_SPAN_SESSION_ID_MAX_EXTERNAL,

    SPECTRUM2_FDB_MID_MAX,
    SPECTRUM2_FDB_MID_START_INDEX,
    SPECTRUM2_FDB_PGT_MAX,
    SPECTRUM2_FDB_UC_ADDRESS_MAX,
    SPECTRUM2_FDB_MC_ADDRESS_MAX,

    SPECTRUM2_FLOODING_TABLE_MAX,
    SPECTRUM2_FLOODING_TABLE_PER_VID_MAX,

    SPECTRUM2_COS_TRAFFIC_CLASS_MAX,
    SPECTRUM2_COS_PORT_ETS_SUB_GROUP_MAX,
    SPECTRUM2_COS_PORT_ETS_GROUP_MAX,
    SPECTRUM2_COS_PORT_PRIO_MAX,
    SPECTRUM2_COS_PORT_COLOR_MAX,
    SPECTRUM2_COS_PORT_ETS_ELEMENTS_NUM,
    SPECTRUM2_COS_BUFFER_64K_SIZE_MAX,
    SPECTRUM2_COS_BUFFER_128K_SIZE_MAX,
    SPECTRUM2_COS_REDECN_DEFAULT_AQS_WEIGHT,
    SPECTRUM2_COS_REDECN_RED_DROP_ENABLED,
    SPECTRUM2_COS_REDECN_SCD,
    SPECTRUM2_COS_REDECN_PROFILES_MAX,
    SPECTRUM2_COS_REDECN_CELL_MULTIPLIER,
    SPECTRUM2_COS_REDECN_DEFAULT_TC_RED_ENABLED,
    SPECTRUM2_COS_REDECN_DEFAULT_TC_ECN_ENABLED,
    SPECTRUM2_SWID_ID_MAX,

    SPECTRUM2_POLICER_POOL_SIZE,
    SPECTRUM2_POLICER_HOST_IFC_POOL_SIZE,
    SPECTRUM2_POLICER_STORM_CONTROL_POOL_SIZE,
    SPECTRUM2_POLICER_BS_MIN_VALUE_PACKETS,
    SPECTRUM2_POLICER_BS_MIN_VALUE_BYTES,
    SPECTRUM2_POLICER_BS_MAX_VALUE,
    SPECTRUM2_KVD_SIZE,
    SPECTRUM2_KVD_HASH_SINGLE_MIN_SIZE,
    SPECTRUM2_KVD_HASH_DOUBLE_MIN_SIZE,
    SPECTRUM2_KVD_GUARANTEED_CAPACITY,
    SPECTRUM2_KVD_LINEAR_VIRTUAL_TABLE_SIZE,

    SPECTRUM2_PGT_SIZE,
    SPECTRUM2_RMPE_SIZE,

    SPECTRUM2_BRIDGE_NUM_MAX,

    SPECTRUM2_BUFFERS_TOTAL_MEMORY,
    SPECTRUM2_DESCRIPTOR_BUFFERS_TOTAL_MEMORY,
    SPECTRUM2_MAX_TOTAL_HEADROOM_SIZE,
    SPECTRUM2_HEADROOM_NUM_BUFF_PER_PORT,
    /* Shared buffers - Condor */
    SPECTRUM2_SHARED_BUFF_NUM_ING_POOL,
    SPECTRUM2_SHARED_BUFF_NUM_EGR_POOL,
    SPECTRUM2_SHARED_BUFF_NUM_MC_POOL,
    SPECTRUM2_SHARED_BUFF_NUM_ING_DESC_POOL,
    SPECTRUM2_SHARED_BUFF_NUM_EGR_DESC_POOL,
    SPECTRUM2_SHARED_BUFF_NUM_MANAGMENT_POOL,
    SPECTRUM2_SHARED_BUFF_TOTAL_NUM_POOL,
    SPECTRUM2_SHARED_BUFF_NUM_EGR_BUFF_PER_PORT,
    SPECTRUM2_SHARED_BUFF_NUM_ING_BUFF_PER_PORT,
    SPECTRUM2_SHARED_BUFF_MAX_POOL_SIZE,
    SPECTRUM2_SHARED_BUFF_PORT_NUM_MAX,
    SPECTRUM2_SHARED_BUFF_TCLASS_NUM_MAX,
    SPECTRUM2_SHARED_BUFF_RET_STATISTICS_NUM_MAX,
    SPECTRUM2_BUFF_UNIT_SIZE,
    SPECTRUM2_SHARED_BUFF_MC_MAX_NUM_PRIO,

    {
        /* RM_SDK_TABLE_TYPE_UC_MAC_E = 0 */
        {SPECTRUM2_FDB_UC_ADDRESS_SDK_TABLE_MIN,
         SPECTRUM2_FDB_UC_ADDRESS_SDK_TABLE_MAX,
         SPECTRUM2_FDB_UC_MAC_HW_ENTRY_COST,
         SPECTRUM2_FDB_UC_MAC_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_MC_MAC_E = 1 */
        {SPECTRUM2_FDB_MC_ADDRESS_SDK_TABLE_MIN,
         SPECTRUM2_FDB_MC_ADDRESS_SDK_TABLE_MAX,
         SPECTRUM2_FDB_MC_MAC_HW_ENTRY_COST,
         SPECTRUM2_FDB_MC_MAC_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV4_UC_E = 2 */
        {SPECTRUM2_ROUTER_IPV4_UC_MAX,
         SPECTRUM2_ROUTER_IPV4_UC_MAX,
         SPECTRUM2_ROUTER_IPV4_UC_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_IPV4_UC_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV6_UC_E 3 */
        {SPECTRUM2_ROUTER_IPV6_UC_MAX,
         SPECTRUM2_ROUTER_IPV6_UC_MAX,
         SPECTRUM2_ROUTER_IPV6_UC_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_IPV6_UC_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV4_MC_E = 4 */
        {SPECTRUM2_ROUTER_IPV4_MC_MAX,
         SPECTRUM2_ROUTER_IPV4_MC_MAX,
         SPECTRUM2_ROUTER_IPV4_MC_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_IPV4_MC_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV6_MC_E = 5 */
        {SPECTRUM2_ROUTER_IPV6_MC_MAX,
         SPECTRUM2_ROUTER_IPV6_MC_MAX,
         SPECTRUM2_ROUTER_IPV6_MC_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_IPV6_MC_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ARP_IPV4_E = 6 */
        {SPECTRUM2_ROUTER_NEIGHS_MAX,
         SPECTRUM2_ROUTER_NEIGHS_MAX,
         SPECTRUM2_ROUTER_ARP_IPV4_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_ARP_IPV4_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ARP_IPV6_E = 7 */
        {SPECTRUM2_ROUTER_NEIGHS_MAX,
         SPECTRUM2_ROUTER_NEIGHS_MAX,
         SPECTRUM2_ROUTER_ARP_IPV6_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_ARP_IPV6_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_L3_MC_REPLICATIONS_E = 8 */
        {SPECTRUM2_ROUTER_IPV4_MC_MAX + SPECTRUM2_ROUTER_IPV6_MC_MAX,
         SPECTRUM2_ROUTER_IPV4_MC_MAX + SPECTRUM2_ROUTER_IPV6_MC_MAX,
         SPECTRUM2_ROUTER_MC_REPLICATIONS_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_MC_REPLICATIONS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ADJACENCY_E = 9 */
        {SPECTRUM2_ROUTER_ADJ_MAX,
         SPECTRUM2_ROUTER_ADJ_MAX,
         SPECTRUM2_ROUTER_ADJACENCY_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_ADJACENCY_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_L2_MC_VECTORS_E = 10 */
        {SPECTRUM2_FDB_MID_MIN,
         SPECTRUM2_FDB_MID_MAX,
         SPECTRUM2_L2_MC_MIDS_HW_ENTRY_COST,
         SPECTRUM2_L2_MC_MIDS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_LAG_VECTORS_E = 11 */
        {SPECTRUM2_LAG_VECTORS_NUM_MAX,
         SPECTRUM2_LAG_VECTORS_NUM_MAX,
         SPECTRUM2_LAG_VECTORS_ENTRY_COST,
         SPECTRUM2_LAG_VECTORS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FLOOD_VECTORS_E = 12 */
        {SPECTRUM2_FLOODING_TABLE_MAX,
         SPECTRUM2_FLOODING_TABLE_MAX,
         SPECTRUM2_L2_MC_FLOODING_HW_ENTRY_COST,
         SPECTRUM2_L2_MC_FLOODING_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},


        /* RM_SDK_TABLE_TYPE_ACL_EXTENDED_ACTIONS_E = 13 */
        {SPECTRUM2_ACL_EXTENDED_ACTIONS_NUM_MIN,
         SPECTRUM2_ACL_EXTENDED_ACTIONS_NUM_MAX,
         SPECTRUM2_ACL_EXTENDED_ACTIONS_HW_ENTRY_COST,
         SPECTRUM2_ACL_EXTENDED_ACTIONS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_PBS_E = 14 */
        {SPECTRUM2_ACL_PBS_ENTRIES_MIN,
         SPECTRUM2_ACL_PBS_ENTRIES_MAX,
         SPECTRUM2_ACL_PBS_HW_ENTRY_COST,
         SPECTRUM2_ACL_PBS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ERIF_LIST_E = 15 */
        {SPECTRUM2_ERIF_LIST_NUM_MIN,
         SPECTRUM2_ERIF_LIST_NUM_MAX,
         SPECTRUM2_ERIF_LIST_HW_ENTRY_COST,
         SPECTRUM2_ERIF_LIST_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_TUNNEL_RTDP_E = 16 */
        {SPECTRUM2_TUNNEL_RTDP_NUM_MIN,
         SPECTRUM2_TUNNEL_RTDP_NUM_MAX,
         SPECTRUM2_TUNNEL_RTDP_HW_ENTRY_COST,
         SPECTRUM2_TUNNEL_RTDP_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},


        /* RM_SDK_TABLE_TYPE_ILM_E = 17 */
        {SPECTRUM2_ILM_NUM_MIN,
         SPECTRUM2_ILM_NUM_MAX,
         SPECTRUM2_ILM_HW_ENTRY_COST,
         SPECTRUM2_ILM_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RPF_GROUP_E = 18 */
        {SPECTRUM2_MC_RPF_GROUP_MIN,
         SPECTRUM2_MC_RPF_GROUP_MAX,
         SPECTRUM2_MC_RPF_GROUP_ENTRY_COST,
         SPECTRUM2_MC_RPF_GROUP_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_VLAN_E = 19 */
        {SPECTRUM2_VLANS_VID_TO_FID_MIN,
         SPECTRUM2_VLANS_VID_TO_FID_MAX,
         SPECTRUM2_VLANS_VID_TO_FID_ENTRY_COST,
         SPECTRUM2_VLANS_VID_TO_FID_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_VPORTS_E = 20 */
        {SPECTRUM2_VPORTS_RQ_VID_TO_FID_MIN,
         SPECTRUM2_VPORTS_RQ_VID_TO_FID_MAX,
         SPECTRUM2_VPORTS_RQ_VID_TO_FID_ENTRY_COST,
         SPECTRUM2_VPORTS_RQ_VID_TO_FID_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FID_E = 21 */
        {SPECTRUM2_FIDS_MIN,
         SPECTRUM2_FIDS_MAX,
         SPECTRUM2_FIDS_ENTRY_COST,
         SPECTRUM2_FIDS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_VNI_E = 22 */
        {SPECTRUM2_VNI_TO_FID_MIN,
         SPECTRUM2_VNI_TO_FID_MAX,
         SPECTRUM2_VNI_TO_FID_ENTRY_COST,
         SPECTRUM2_VNI_TO_FID_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_TUNNEL_IPV6_FDB_E = 23 */
        {SPECTRUM2_TUNNEL_IPV6_FDB_MIN,
         SPECTRUM2_TUNNEL_IPV6_FDB_MAX,
         SPECTRUM2_TUNNEL_IPV6_FDB_ENTRY_COST,
         SPECTRUM2_TUNNEL_IPV6_FDB_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_MPLS_NHLFE_E = 24 */
        {SPECTRUM2_MPLS_NHLFE_MIN,
         SPECTRUM2_MPLS_NHLFE_MAX,
         SPECTRUM2_MPLS_NHLFE_ENTRY_COST,
         SPECTRUM2_MPLS_NHLFE_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_PB_MPLS_ILM_E = 25 */
        {SPECTRUM2_PB_MPLS_ILM_MIN,
         SPECTRUM2_PB_MPLS_ILM_MAX,
         SPECTRUM2_PB_MPLS_ILM_ENTRY_COST,
         SPECTRUM2_PB_MPLS_ILM_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_NVE_IPV6_LIST_E = 26 */
        {SPECTRUM2_TUNNEL_NVE_IPV6_LIST_MIN,
         SPECTRUM2_TUNNEL_NVE_IPV6_LIST_MAX,
         SPECTRUM2_TUNNEL_NVE_IPV6_LIST_ENTRY_COST,
         SPECTRUM2_TUNNEL_NVE_IPV6_LIST_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_E = 27 */
        {SPECTRUM2_ACL_REGIONS_MIN,
         SPECTRUM2_ACL_REGIONS_MAX,
         SPECTRUM2_ACL_REGIONS_ENTRY_COST,
         SPECTRUM2_ACL_REGIONS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RIPS_E = 28 */
        {SPECTRUM2_IPV6_MGR_RIPS_NUM_MIN,
         SPECTRUM2_IPV6_MGR_RIPS_NUM_MAX,
         SPECTRUM2_IPV6_MGR_RIPS_HW_ENTRY_COST,
         SPECTRUM2_IPV6_MGR_RIPS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_IGMP_V3_IPv4_E = 29 */
        {SPECTRUM2_IGMP_V3_NUM_MIN,
         SPECTRUM2_IGMP_V3_NUM_MAX,
         SPECTRUM2_IGMP_V3_HW_ENTRY_COST,
         SPECTRUM2_IGMP_V3_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_IGMP_V3_IPv6_E = 30 */
        {SPECTRUM2_IGMP_V3_NUM_MIN,
         SPECTRUM2_IGMP_V3_NUM_MAX,
         SPECTRUM2_IGMP_V3_HW_ENTRY_COST,
         SPECTRUM2_IGMP_V3_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_DECAP_RULES_IPv4_E = 31 */
        {SPECTRUM2_DECAP_RULES_NUM_MIN,
         SPECTRUM2_DECAP_RULES_NUM_MAX,
         SPECTRUM2_DECAP_RULES_HW_ENTRY_COST,
         SPECTRUM2_DECAP_RULES_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_DECAP_RULES_IPv6_E = 32 */
        {SPECTRUM2_DECAP_RULES_NUM_MIN,
         SPECTRUM2_DECAP_RULES_NUM_MAX,
         SPECTRUM2_DECAP_RULES_HW_ENTRY_COST,
         SPECTRUM2_DECAP_RULES_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_RULES_TWO_KEY_BLOCK_E = 33 */
        {SPECTRUM2_ACL_RULES_TWO_KEY_BLOCK_NUM_MIN,
         SPECTRUM2_ACL_RULES_TWO_KEY_BLOCK_NUM_MAX,
         SPECTRUM2_ACL_RULES_TWO_KEY_BLOCK_HW_ENTRY_COST,
         SPECTRUM2_ACL_RULES_TWO_KEY_BLOCK_HW_TABLE_TYPE,
         SPECTRUM2_ACL_RULES_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_RULES_FOUR_KEY_BLOCK_E = 34 */
        {SPECTRUM2_ACL_RULES_FOUR_KEY_BLOCK_NUM_MIN,
         SPECTRUM2_ACL_RULES_FOUR_KEY_BLOCK_NUM_MAX,
         SPECTRUM2_ACL_RULES_FOUR_KEY_BLOCK_HW_ENTRY_COST,
         SPECTRUM2_ACL_RULES_FOUR_KEY_BLOCK_HW_TABLE_TYPE,
         SPECTRUM2_ACL_RULES_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_RULES_SIX_KEY_BLOCK_E = 35 */
        {SPECTRUM2_ACL_RULES_SIX_KEY_BLOCK_NUM_MIN,
         SPECTRUM2_ACL_RULES_SIX_KEY_BLOCK_NUM_MAX,
         SPECTRUM2_ACL_RULES_SIX_KEY_BLOCK_HW_ENTRY_COST,
         SPECTRUM2_ACL_RULES_SIX_KEY_BLOCK_HW_TABLE_TYPE,
         SPECTRUM2_ACL_RULES_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RIF_COUNTER_BASIC_E = 36 */
        {SPECTRUM2_RIF_COUNTER_BASIC_NUM_MIN,
         SPECTRUM2_RIF_COUNTER_BASIC_NUM_MAX,
         SPECTRUM2_RIF_COUNTER_BASIC_HW_ENTRY_COST,
         SPECTRUM2_RIF_COUNTER_BASIC_HW_TABLE_TYPE,
         SPECTRUM2_RIF_COUNTER_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RIF_COUNTER_MIXED_E = 37 */
        {0,
         0,
         0,
         0,
         0,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RIF_COUNTER_ENHANCED_E = 38 */
        {SPECTRUM2_RIF_COUNTER_ENHANCED_NUM_MIN,
         SPECTRUM2_RIF_COUNTER_ENHANCED_NUM_MAX,
         SPECTRUM2_RIF_COUNTER_ENHANCED_HW_ENTRY_COST,
         SPECTRUM2_RIF_COUNTER_ENHANCED_HW_TABLE_TYPE,
         SPECTRUM2_RIF_COUNTER_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FLOW_COUNTER_E = 39 */
        {SPECTRUM2_FLOW_COUNTER_NUM_MIN,
         SPECTRUM2_FLOW_COUNTER_NUM_MAX,
         SPECTRUM2_FLOW_COUNTER_HW_ENTRY_COST,
         SPECTRUM2_FLOW_COUNTER_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_GROUPS_E = 40 */
        {SPECTRUM2_ACL_GROUPS_NUM_MIN,
         SPECTRUM2_ACL_GROUPS_NUM_MAX,
         SPECTRUM2_ACL_GROUPS_HW_ENTRY_COST,
         SPECTRUM2_ACL_GROUPS_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_INTERNAL_E = 41 : Divider between int & ext*/
        {0,
         0,
         0,
         0,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_RULES_E = 42 */
        {SPECTRUM2_ACL_RULES_NUM_MIN,
         SPECTRUM2_ACL_RULES_NUM_MAX,
         SPECTRUM2_ACL_RULES_HW_ENTRY_COST,
         SPECTRUM2_ACL_RULES_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_DECAP_RULES_E = 43 */
        {SPECTRUM2_DECAP_RULES_NUM_MIN,
         SPECTRUM2_DECAP_RULES_NUM_MAX,
         SPECTRUM2_DECAP_RULES_HW_ENTRY_COST,
         SPECTRUM2_DECAP_RULES_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_IGMP_V3_E = 44 */
        {SPECTRUM2_IGMP_V3_NUM_MIN,
         SPECTRUM2_IGMP_V3_NUM_MAX,
         SPECTRUM2_IGMP_V3_HW_ENTRY_COST,
         SPECTRUM2_IGMP_V3_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_ONE_KEY_BLOCK_E = 45 */
        {SPECTRUM2_A_TCAM_ONE_KEY_BLOCK_MIN,
         SPECTRUM2_A_TCAM_ONE_KEY_BLOCK_MAX,
         SPECTRUM2_A_TCAM_ONE_KEY_BLOCK_ENTRY_COST,
         SPECTRUM2_A_TCAM_ONE_KEY_BLOCK_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_TWO_KEYS_BLOCK_E = 46 */
        {SPECTRUM2_A_TCAM_TWO_KEYS_BLOCK_MIN,
         SPECTRUM2_A_TCAM_TWO_KEYS_BLOCK_MAX,
         SPECTRUM2_A_TCAM_TWO_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM2_A_TCAM_TWO_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_FOUR_KEYS_BLOCK_E = 47 */
        {SPECTRUM2_A_TCAM_FOUR_KEYS_BLOCK_MIN,
         SPECTRUM2_A_TCAM_FOUR_KEYS_BLOCK_MAX,
         SPECTRUM2_A_TCAM_FOUR_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM2_A_TCAM_FOUR_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_SIX_KEYS_BLOCK_E = 48 */
        {SPECTRUM2_A_TCAM_SIX_KEYS_BLOCK_MIN,
         SPECTRUM2_A_TCAM_SIX_KEYS_BLOCK_MAX,
         SPECTRUM2_A_TCAM_SIX_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM2_A_TCAM_SIX_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FCF_FORWORDING_E = 49 */
        {SPECTRUM2_FCF_RULES_MAX,
         SPECTRUM2_FCF_RULES_MAX,
         SPECTRUM2_FCF_FORWORDING_HW_ENTRY_COST,
         SPECTRUM2_FCF_FORWORDING_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RMID_MANAGER_E = 50 */
        {SPECTRUM2_RMID_NUM_MIN,
         SPECTRUM2_RMID_NUM_MAX,
         SPECTRUM2_RMID_HW_ENTRY_COST,
         SPECTRUM2_RMID_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_NVE_MC_LIST_E = 51 */
        {SPECTRUM2_NVE_MC_LIST_NUM_MIN,
         SPECTRUM2_NVE_MC_LIST_NUM_MAX,
         SPECTRUM2_NVE_MC_LIST_HW_ENTRY_COST,
         SPECTRUM2_NVE_MC_LIST_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_SMID_MANAGER_E = 52 */
        {SPECTRUM2_SMID_NUM_MIN,
         SPECTRUM2_SMID_NUM_MAX,
         SPECTRUM2_SMID_HW_ENTRY_COST,
         SPECTRUM2_SMID_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV4_MC_SYSTEM_E = 53 */
        {SPECTRUM2_ROUTER_IPV4_MC_SYSTEM_MIN,
         SPECTRUM2_ROUTER_IPV4_MC_SYSTEM_MAX,
         SPECTRUM2_ROUTER_IPV4_MC_SYSTEM_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_IPV4_MC_SYSTEM_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV6_MC_SYSTEM_E = 54 */
        {SPECTRUM2_ROUTER_IPV6_MC_SYSTEM_MIN,
         SPECTRUM2_ROUTER_IPV6_MC_SYSTEM_MAX,
         SPECTRUM2_ROUTER_IPV6_MC_SYSTEM_HW_ENTRY_COST,
         SPECTRUM2_ROUTER_IPV6_MC_SYSTEM_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},


        /* RM_SDK_TABLE_TYPE_RIF_COUNTER_E = 55 */
        {SPECTRUM2_RIF_COUNTER_NUM_MIN,
         SPECTRUM2_RIF_COUNTER_NUM_MAX,
         SPECTRUM2_RIF_COUNTER_HW_ENTRY_COST,
         SPECTRUM2_RIF_COUNTER_HW_TABLE_TYPE,
         SPECTRUM2_INVALID_SW_TABLE_TYPE,
         FALSE},
    },

    {
        /* No TCAM management by RM in Spectrum */
        { TRUE, UNLIMITED_SDK_TABLE_SIZE },
        { FALSE, 0 },
        { FALSE, 0 }, /*kvd hash size should be configured by user size*/
        { FALSE, 0 }, /*kvd linear size should be configured by user size */
        { FALSE, 0 },
        { FALSE, 0 },
        { TRUE, SPECTRUM2_ACL_GROUPS_NUM_MAX },
        { FALSE, 0 }, /* PGT size should be configured by user size */
        { TRUE, (SPECTRUM2_CNTR_BANK_PAIRS * SPECTRUM2_CNTR_LINES_PER_BANK * 2)},
        { TRUE, SPECTRUM2_ACL_REGIONS_MAX },
    },

    /*16*/
    SPECTRUM2_CNTR_BANK_PAIRS,
    SPECTRUM2_CNTR_LINES_PER_BANK,
    SPECTRUM2_CNTR_LINES_FLOW_PKT,
    SPECTRUM2_CNTR_TYPE_FLOW_PKT,
    SPECTRUM2_CNTR_LINES_FLOW_BYTE,
    SPECTRUM2_CNTR_TYPE_FLOW_BYTE,
    SPECTRUM2_CNTR_LINES_FLOW_BOTH,
    SPECTRUM2_CNTR_TYPE_FLOW_BOTH,
    SPECTRUM2_CNTR_LINES_RIF_BASIC,
    SPECTRUM2_CNTR_TYPE_RIF_BASIC,
    SPECTRUM2_CNTR_LINES_RIF_MIXED_1,
    SPECTRUM2_CNTR_TYPE_RIF_MIXED_1,
    SPECTRUM2_CNTR_LINES_RIF_MIXED_2,
    SPECTRUM2_CNTR_TYPE_RIF_MIXED_2,
    SPECTRUM2_CNTR_LINES_RIF_ENHANCED,
    SPECTRUM2_CNTR_TYPE_RIF_ENHANCED,

    /*10*/
    SPECTRUM2_IB_PKEY_TABLE_SIZE,

    SPECTRUM2_IB_ROUTER_LIDS_MAX,
    SPECTRUM2_IB_ROUTER_MC_LIDS_NUM_MAX,
    SPECTRUM2_IB_ROUTER_UC_LID_NUM_MAX,

    /*1*/
    SPECTRUM2_IB_PORTS_MAX,

    /*5*/
    SPECTRUM2_NUM_HW_CPU_INGRESS_TCLASS_MAX,
    SPECTRUM2_NUM_HW_DR_PATHS_MAX,
    SPECTRUM2_NUM_HW_TRAP_GROUPS_MAX,
    SPECTRUM2_TRAP_GROUP_PRIORITY_MIN,
    SPECTRUM2_TRAP_GROUP_PRIORITY_MAX,
    SPECTRUM2_NUM_RDQS_MAX,
    SPECTRUM2_NUM_CPU_TCLASS,

    /* Tunnel */
    SPECTRUM2_NUM_MAX_TUNNEL_IPINIP,
    SPECTRUM2_NUM_MAX_TUNNEL_NVE,
    SPECTRUM2_TUNNEL_NVE_GS_FLOOD_MAX,
    SPECTRUM2_TUNNEL_NVE_GS_MC_MAX,
    SPECTRUM2_TUNNEL_TNUMT_IPV6_RECORDS_MAX,

    /* IGMP V3 */
    SPECTRUM2_IGMP_V3_NUM_MAX_ENTRIES,

    /*Telemetry */
    SPECTRUM2_TELE_HISTOGRAM_NUM_MAX,
    SPECTRUM2_TELE_HISTOGRAM_QUEUE_DEPTH_BINS,
    SPECTRUM2_TELE_HISTOGRAM_SAMPLE_TIME_RESOLUTION_MAX,
    SPECTRUM2_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_MIN,
    SPECTRUM2_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_MAX,
    SPECTRUM2_TELE_HISTOGRAM_DATA_BIN_BITS_MAX,
    SPECTRUM2_TELE_HISTOGRAM_MIN_BOUNDARY_MIN,
    SPECTRUM2_TELE_THRESHOLD_THR_MAX,
};

/************************************************
 *  Function declarations
 ***********************************************/
#endif /*__SPECTRUM2_H__*/
