/*
 *  Copyright (C) 2010-2018. Mellanox Technologies, Ltd. ALL RIGHTS RESERVED.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License"); you may
 *    not use this file except in compliance with the License. You may obtain
 *    a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 *    THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR
 *    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 *    LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS
 *    FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 *    See the Apache Version 2.0 License for specific language governing
 *    permissions and limitations under the License.
 *
 */


#ifndef SX_TRAP_ID_H_
#define SX_TRAP_ID_H_

#include <complib/cl_types.h>
#include <sx/sdk/sx_check.h>
#include <sx/sxd/sxd_trap_id.h>
#include <sx/sdk/obsolete.h>

/**
 * sx_trap_id_t enumerated type is used to note the supported traps
 */
typedef enum sx_trap_id {
    /* GENERAL */
    SX_TRAP_ID_GENERAL_FDB = SXD_TRAP_ID_GENERAL_FDB,       /**< FDB */
    SX_TRAP_ID_GENERAL_DR_IPC = SXD_TRAP_ID_GENERAL_DR_IPC,     /**< DR IPC */
    SX_TRAP_ID_GENERAL_DR_RES = SXD_TRAP_ID_GENERAL_DR_RES,     /**< DR RES */
    SX_TRAP_ID_GENERAL_ETH_EMAD = SXD_TRAP_ID_GENERAL_ETH_EMAD, /**< ETHERNET EMAD */
    SX_TRAP_ID_FDB = SXD_TRAP_ID_FDB,               /**< FDB TRAP */

    /* EVENTS */
    SX_TRAP_ID_PUDE = SXD_TRAP_ID_PUDE,             /**< port up/down*/
    SX_TRAP_ID_PMPE = SXD_TRAP_ID_PMPE,             /**< port module plug / unplug */
    SX_TRAP_ID_FLAE = SXD_TRAP_ID_FLAE,             /**< FDB learning and aging event */
    SX_TRAP_ID_TMPW = SXD_TRAP_ID_TMPW,             /**< Temperature warning event */
    SX_TRAP_ID_CPUWD = SXD_TRAP_ID_CPUWD,        /**< CPU Watchdog Event */
    SX_TRAP_ID_FORE = SXD_TRAP_ID_FORE,              /**< Fan Out of Rang event generated by hardware */
    SX_TRAP_ID_BER_MONITOR = SXD_TRAP_ID_PPMBE,      /**< BER monitor trap */

    /* SWITCHX SDK EVENTS */
    SX_TRAP_ID_SIGNAL = SXD_TRAP_ID_SIGNAL,             /**< signal trap for SDK internal use */
    SX_TRAP_ID_NEW_DEVICE_ADD = 0x201,                  /**< device add event */
    SX_TRAP_ID_MAC_LEARNING_EVENT = 0x202,              /**< mac learning event */
    SX_TRAP_ID_MAC_AGING_EVENT = 0x203,                 /**< mac aging event */
    SX_TRAP_ID_NEED_TO_RESOLVE_ARP = 0x204,             /**< need to resolve ARP  */
    SX_TRAP_ID_NO_NEED_TO_RESOLVE_ARP = 0x205,          /**< no need to resolve ARP  */
    SX_TRAP_ID_FDB_EVENT = 0x206,                       /**< FDB event */
    SX_TRAP_ID_RM_SDK_TABLE_THRESHOLD_EVENT = 0x207,
    SX_TRAP_ID_RM_HW_TABLE_THRESHOLD_EVENT = 0x208,
    SX_TRAP_ID_FDB_SRC_MISS = 0x209,                    /**< FDB SRC MISS trap */
    SX_TRAP_ID_ROUTER_NEIGH_ACTIVITY = 0x20a,
    SX_TRAP_ID_ASYNC_API_COMPLETE_EVENT = 0x20b,
    SX_TRAP_ID_ROUTER_MC_ACTIVITY = 0x20c,              /**< router mc activity */
    SX_TRAP_ID_FDB_IP_ADDR_ACTIVITY = 0x20d,            /**< fdb ip addr activity */
    SX_TRAP_ID_TRANSACTION_ERROR = SXD_TRAP_ID_TRANSACTION_ERROR,               /**< error in transaction mode */

    /* ETHERNET L2 */
    SX_TRAP_ID_ETH_L2_STP = SXD_TRAP_ID_ETH_L2_STP,                 /**< ETHERNET L2 STP */
    SX_TRAP_ID_ETH_L2_LACP = SXD_TRAP_ID_ETH_L2_LACP,               /**< ETHERNET L2 LACP */
    SX_TRAP_ID_ETH_L2_EAPOL = SXD_TRAP_ID_ETH_L2_EAPOL,             /**< ETHERNET L2 EAPOL */
    SX_TRAP_ID_ETH_L2_LLDP = SXD_TRAP_ID_ETH_L2_LLDP,               /**< ETHERNET L2 LLDP */
    SX_TRAP_ID_ETH_L2_MMRP = SXD_TRAP_ID_ETH_L2_MMRP,               /**< ETHERNET L2 MMRP */
    SX_TRAP_ID_ETH_L2_MVRP = SXD_TRAP_ID_ETH_L2_MVRP,               /**< ETHERNET L2 MVRP */
    SX_TRAP_ID_ETH_L2_RPVST = SXD_TRAP_ID_ETH_L2_RPVST,             /**< ETHERNET L2 RPVST */
    SX_TRAP_ID_ETH_L2_IGMP_TYPE_QUERY = SXD_TRAP_ID_ETH_L2_IGMP_TYPE_QUERY,     /**< ETHERNET L2 IGMP QUERY */
    SX_TRAP_ID_ETH_L2_IGMP_TYPE_V1_REPORT = SXD_TRAP_ID_ETH_L2_IGMP_TYPE_V1_REPORT, /**< ETHERNET L2 IGMP V1_REPORT */
    SX_TRAP_ID_ETH_L2_IGMP_TYPE_V2_REPORT = SXD_TRAP_ID_ETH_L2_IGMP_TYPE_V2_REPORT, /**< ETHERNET L2 IGMP V2_REPORT */
    SX_TRAP_ID_ETH_L2_IGMP_TYPE_V2_LEAVE = SXD_TRAP_ID_ETH_L2_IGMP_TYPE_V2_LEAVE,   /**< ETHERNET L2 IGMP V2_LEAVE */
    SX_TRAP_ID_ETH_L2_IGMP_TYPE_V3_REPORT = SXD_TRAP_ID_ETH_L2_IGMP_TYPE_V3_REPORT, /**< ETHERNET L2 IGMP V3_REPORT */
    SX_TRAP_ID_ETH_L2_DHCP = SXD_TRAP_ID_ETH_L2_DHCP,               /**< ETHERNET DHCP */
    SX_TRAP_ID_ETH_L2_UDLD = SXD_TRAP_ID_ETH_L2_UDLD,               /**< ETHERNET UDLD */
    SX_TRAP_ID_ETH_L2_PACKET_SAMPLING = SXD_TRAP_ID_ETH_L2_PKT_SAMPLE,      /**< ETHERNET L2 PACKET_SAMPLING */
    SX_TRAP_ID_ETH_L2_RARP_OPCODES = SXD_TRAP_ID_RARP_OPCODES,      /**< ETHERNET L2 RARP OPCODES */

    /* FCoE */
    SX_TRAP_ID_FCOE_FIP = SXD_TRAP_ID_FCOE_FIP,         /**< FCOE FIP */

    /* Telemetry threshold crossing */
    SX_TRAP_ID_QUEUE_THRESHOLD_CROSSED = SXD_TRAP_ID_SB_CONG_TX_PORT,

    /* ECN */
    SX_TRAP_ID_DECAP_ECN0 = SXD_TRAP_ID_DECAP_ECN0,     /**< NVE/IPinIP/MPLS ECN0 */
    SX_TRAP_ID_DECAP_ECN1 = SXD_TRAP_ID_DECAP_ECN1,     /**< NVE/IPinIP/MPLS ECN1 */

    /* Router */
    SX_TRAP_ID_ARP_REQUEST = SXD_TRAP_ID_ARP_REQUEST,       /**< ARP REQUEST */
    SX_TRAP_ID_ARP_RESPONSE = SXD_TRAP_ID_ARP_RESPONSE,     /**< ARP RESPONSE */
    SX_TRAP_ID_ETH_L3_MTUERROR = SXD_TRAP_ID_ETH_L3_MTUERROR,   /**< Packet exceed router interface MTU */
    SX_TRAP_ID_ETH_L3_TTLERROR = SXD_TRAP_ID_ETH_L3_TTLERROR,   /**< Time To Live (TTL) Expired */
    SX_TRAP_ID_ETH_L3_LBERROR = SXD_TRAP_ID_ETH_L3_LBERROR,
    SX_TRAP_ID_OSPF = SXD_TRAP_ID_OSPF,
    SX_TRAP_ID_RIP_V1 = SXD_TRAP_ID_RIP_V1,
    SX_TRAP_ID_RIP_V2 = SXD_TRAP_ID_RIP_V2,
    SX_TRAP_ID_PIM = SXD_TRAP_ID_PIM,
    SX_TRAP_ID_VRRP = SXD_TRAP_ID_VRRP,
    SX_TRAP_ID_VRRP_IPV6 = SXD_TRAP_ID_VRRP_IPV6,
    SX_TRAP_ID_RESERVED_MC = SXD_TRAP_ID_RESERVED_MC,
    SX_TRAP_ID_IPBC = SXD_TRAP_ID_IPBC,
    SX_TRAP_ID_ETH_L3_RPF = SXD_TRAP_ID_ETH_L3_RPF,
    SX_TRAP_ID_ETH_L3_ASSERT = SXD_TRAP_ID_ETH_L3_ASSERT,
    SX_TRAP_ID_L3_UC_IP_BASE = SXD_TRAP_ID_IPTRAP_MIN,
    SX_TRAP_ID_IP2ME = SXD_TRAP_ID_IP2ME,
    SX_TRAP_ID_L3_MC_IP_BASE = SX_TRAP_ID_L3_UC_IP_BASE + SXD_TRAP_GROUP_MAX + 1,
    SX_TRAP_ID_L3_NEIGH_IP_BASE = SX_TRAP_ID_L3_MC_IP_BASE + SXD_TRAP_GROUP_MAX + 1,
    SX_TRAP_ID_HOST_MISS_IPV4 = SXD_TRAP_ID_HOST_MISS_IPV4,
    SX_TRAP_ID_HOST_MISS_IPV6 = SXD_TRAP_ID_HOST_MISS_IPV6,
    SX_TRAP_ID_BFD_IPV4 = SXD_TRAP_ID_BFD_IPV4,
    SX_TRAP_ID_BFD_IPV6 = SXD_TRAP_ID_BFD_IPV6,
    SX_TRAP_ID_SSH_IPV4 = SXD_TRAP_ID_SSH_IPV4,
    SX_TRAP_ID_SSH_IPV6 = SXD_TRAP_ID_SSH_IPV6,
    SX_TRAP_ID_PING_IPV4 = SXD_TRAP_ID_PING_IPV4,
    SX_TRAP_ID_PING_IPV6 = SXD_TRAP_ID_PING_IPV6,
    SX_TRAP_ID_ROUTER_ALERT_IPV4 = SXD_TRAP_ID_ROUTER_ALERT_IPV4,
    SX_TRAP_ID_IPV4_DHCP = SXD_TRAP_ID_IPV4_DHCP,
    SX_TRAP_ID_SNMP_IPV4 = SXD_TRAP_ID_IPV4_SNMP,
    SX_TRAP_ID_SNMP_IPV6 = SXD_TRAP_ID_IPV6_SNMP,
    SX_TRAP_ID_ROUTER_ARPBC = SXD_TRAP_ID_ROUTER_ARPBC,
    SX_TRAP_ID_ROUTER_APRUC = SXD_TRAP_ID_ROUTER_APRUC,
    SX_TRAP_ID_ROUTER_RARP_OPCODES = SXD_TRAP_ID_ROUTER_RARP_OPCODES,


    /*IPv6 L3*/
    SX_TRAP_ID_IPV6_UNSPECIFIED_ADDRESS = SXD_TRAP_ID_IPV6_UNSPECIFIED_ADDRESS,
    SX_TRAP_ID_IPV6_LINK_LOCAL_DST = SXD_TRAP_ID_IPV6_LINK_LOCAL_DST,
    SX_TRAP_ID_IPV6_LINK_LOCAL_SRC = SXD_TRAP_ID_IPV6_LINK_LOCAL_SRC,
    SX_TRAP_ID_IPV6_ALL_NODES_LINK = SXD_TRAP_ID_IPV6_ALL_NODES_LINK,
    SX_TRAP_ID_IPV6_ROUTER_SOLICITATION = SXD_TRAP_ID_IPV6_ROUTER_SOLICITATION,
    SX_TRAP_ID_IPV6_ROUTER_ADVERTISEMENT = SXD_TRAP_ID_IPV6_ROUTER_ADVERTISEMENT,
    SX_TRAP_ID_IPV6_NEIGHBOR_SOLICITATION = SXD_TRAP_ID_IPV6_NEIGHBOR_SOLICITATION,
    SX_TRAP_ID_IPV6_NEIGHBOR_ADVERTISEMENT = SXD_TRAP_ID_IPV6_NEIGHBOR_ADVERTISEMENT,
    SX_TRAP_ID_IPV6_REDIRECTION = SXD_TRAP_ID_IPV6_REDIRECTION,
    SX_TRAP_ID_IPV6_ALL_ROUTERS_LINK = SXD_TRAP_ID_IPV6_ALL_ROUTERS_LINK,
    SX_TRAP_ID_IPV6_OSPF = SXD_TRAP_ID_IPV6_OSPF,
    SX_TRAP_ID_IPV6_DHCP = SXD_TRAP_ID_IPV6_DHCP,
    SX_TRAP_ID_ROUTER_ALERT_IPV6 = SXD_TRAP_ID_ROUTER_ALERT_IPV6,

    /*IPv6 L2*/
    SX_TRAP_ID_IPV6_MLD_V1_V2 = SXD_TRAP_ID_IPV6_MLD_V1_V2,
    SX_TRAP_ID_IPV6_MLD_V1_REPORT = SXD_TRAP_ID_IPV6_MLD_V1_REPORT,
    SX_TRAP_ID_IPV6_MLD_V1_DONE = SXD_TRAP_ID_IPV6_MLD_V1_DONE,
    SX_TRAP_ID_IPV6_MLD_V2_REPORT = SXD_TRAP_ID_IPV6_MLD_V2_REPORT,

    /* InfiniBand */
    SX_TRAP_ID_INFINIBAND_QP0 = SXD_TRAP_ID_INFINIBAND_QP0,             /**< IB QP0 */
    SX_TRAP_ID_INFINIBAND_QP1 = SXD_TRAP_ID_INFINIBAND_QP1,             /**< IB QP1 */
    SX_TRAP_ID_INFINIBAND_OTHER_QPS = SXD_TRAP_ID_INFINIBAND_OTHER_QPS,     /**< IB OTHERS */
    SX_TRAP_ID_INFINIBAND_EXTERNAL_SMA = SXD_TRAP_ID_INFINIBAND_EXTERNAL_SMA,   /**< IB External SMA */
    SX_TRAP_ID_INFINIBAND_IN_NV_ACCESS_REG = SXD_TRAP_ID_INFINIBAND_IN_NV_ACCESS_REG, /**< IB VS-MAD with ACCESS_REG with one of the NV Config registers */

    SX_TRAP_ID_IPTRAP_MIN = SX_TRAP_ID_L3_UC_IP_BASE,         /**< IPTRAP MIN */
    SX_TRAP_ID_IPTRAP_MAX = SX_TRAP_ID_L3_NEIGH_IP_BASE + SXD_TRAP_GROUP_MAX,   /**< IPTRAP MAX */

    /* ACL */
    SX_TRAP_ID_ACL_MIN = SX_TRAP_ID_IPTRAP_MAX + 1,     /**< ACL MIN */
    SX_TRAP_ID_ACL_LOG = SX_TRAP_ID_ACL_MIN,
    SX_TRAP_ID_ACL_MAX = SXD_TRAP_ID_ACL_MAX,           /**< ACL MAX */

    /* Decap Table */
    SX_TRAP_ID_DECAP_TABLE_0 = SX_TRAP_ID_ACL_MAX,
    SX_TRAP_ID_DECAP_TABLE_1 = SX_TRAP_ID_ACL_MAX - 1,

    /* IGMP V3 */
    SX_TRAP_ID_IGMP_V3_P1 = SX_TRAP_ID_ACL_MAX - 2,
    SX_TRAP_ID_IGMP_V3_P0 = SX_TRAP_ID_ACL_MAX - 3,
    SX_TRAP_ID_IGMP_V3_BASE = SX_TRAP_ID_IGMP_V3_P0,

    /* MPLS */
    SX_TRAP_ID_MPLS_ILM_MISS = SXD_TRAP_ID_MPLS_ILM_MISS,
    SX_TRAP_ID_MPLS_LDP = SXD_TRAP_ID_MPLS_LDP,
    SX_TRAP_ID_MPLS_LB_LSP_PING = SXD_TRAP_ID_MPLS_LB_LSP_PING,
    SX_TRAP_ID_MPLS_ILM0 = SXD_TRAP_ID_MPLS_ILM0,
    SX_TRAP_ID_MPLS_ILM1 = SXD_TRAP_ID_MPLS_ILM1,
    SX_TRAP_ID_MPLS_NHLFE0 = SXD_TRAP_ID_MPLS_NHLFE0,
    SX_TRAP_ID_MPLS_NHLFE1 = SXD_TRAP_ID_MPLS_NHLFE1,

    /* BGP */
    SX_TRAP_ID_IPV4_BGP = SXD_TRAP_ID_IPV4_BGP,
    SX_TRAP_ID_IPV6_BGP = SXD_TRAP_ID_IPV6_BGP,

    /* SPAN */
    SX_TRAP_ID_MIRROR = SXD_TRAP_ID_MIRROR,

    /* Tunneling */
    SX_TRAP_ID_DECAP_ENCAP = SXD_TRAP_ID_DECAP_ENCAP,
    SX_TRAP_ID_IPIP_DECAP_ERROR = SXD_TRAP_ID_IPIP_DECAP_ERROR,
    SX_TRAP_ID_IPIP_ERROR = SXD_TRAP_ID_IPIP_ERROR,
    SX_TRAP_ID_NVE_DECAP_ARP = SXD_TRAP_ID_NVE_DECAP_ARP,
    SX_TRAP_ID_NVE_DECAP_TAG_ERROR = SXD_TRAP_ID_NVE_DECAP_TAG_ERROR,
    SX_TRAP_ID_NVE_IPV4_DHCP = SXD_TRAP_ID_NVE_IPV4_DHCP,
    SX_TRAP_ID_NVE_IPV6_DHCP = SXD_TRAP_ID_NVE_IPV6_DHCP,
    SX_TRAP_ID_NVE_DECAP_FRAG_ERROR = SXD_TRAP_ID_NVE_DECAP_FRAG_ERROR,
    SX_TRAP_ID_NVE_ENCAP_ARP = SXD_TRAP_ID_NVE_ENCAP_ARP,

    /* PTP traps */
    SX_TRAP_ID_PTP_EVENT = SXD_TRAP_ID_PTP_PTP0,
    SX_TRAP_ID_PTP_GENERAL = SXD_TRAP_ID_PTP_PTP1,
    SX_TRAP_ID_PTP_ING_EVENT = SXD_TRAP_ID_PTP_ING_EVENT,
    SX_TRAP_ID_PTP_EGR_EVENT = SXD_TRAP_ID_PTP_EGR_EVENT,

    /* Discards */
    SX_TRAP_ID_DISCARD_ING_PACKET = SXD_TRAP_ID_DISCARD_ING_PACKET,
    SX_TRAP_ID_DISCARD_ING_SWITCH = SXD_TRAP_ID_DISCARD_ING_SWITCH,
    SX_TRAP_ID_DISCARD_LOOKUP_SWITCH = SXD_TRAP_ID_DISCARD_LOOKUP_SWITCH,
    SX_TRAP_ID_DISCARD_ING_ROUTER = SXD_TRAP_ID_DISCARD_ING_ROUTER,
    SX_TRAP_ID_DISCARD_ING_LSR = SXD_TRAP_ID_DISCARD_ING_LSR,
    SX_TRAP_ID_DISCARD_ROUTER = SXD_TRAP_ID_DISCARD_ROUTER,
    SX_TRAP_ID_DISCARD_LSR = SXD_TRAP_ID_DISCARD_LSR,
    SX_TRAP_ID_DISCARD_DEC = SXD_TRAP_ID_DISCARD_DEC,
    SX_TRAP_ID_DISCARD_OVERLAY_SWITCH = SXD_TRAP_ID_DISCARD_OVERLAY_SWITCH,
    SX_TRAP_ID_DISCARD_ISOLATION = SXD_TRAP_ID_DISCARD_ISOLATION,
    SX_TRAP_ID_DISCARD_NON_ROUTED = SXD_TRAP_ID_DISCARD_NON_ROUTED,
    SX_TRAP_ID_DISCARD_EGR_LSR = SXD_TRAP_ID_DISCARD_EGR_LSR,
    SX_TRAP_ID_DISCARD_MC_SCOPE = SXD_TRAP_ID_DISCARD_MC_SCOPE,
    SX_TRAP_ID_DISCARD_ROUTER2 = SXD_TRAP_ID_DISCARD_ROUTER2,
    SX_TRAP_ID_DISCARD_ROUTER3 = SXD_TRAP_ID_DISCARD_ROUTER3,
    SX_TRAP_ID_DISCARD_LSR2 = SXD_TRAP_ID_DISCARD_LSR2,
    SX_TRAP_ID_DISCARD_LSR3 = SXD_TRAP_ID_DISCARD_LSR3,

    /* User defined trap ID's*/

    SX_TRAP_ID_USER_BASE = SXD_TRAP_ID_IP2ME_CUSTOM0,
    SX_TRAP_ID_USER_MAX = SXD_TRAP_ID_IP2ME_CUSTOM1,

    SX_TRAP_ID_USER_ETH_CONF_TYPE0 = SXD_TRAP_ID_ETH_CONF_TYPE0,
    SX_TRAP_ID_USER_ETH_CONF_TYPE1 = SXD_TRAP_ID_ETH_CONF_TYPE1,
    SX_TRAP_ID_USER_ICMPV6_CONF_TYPE0 = SXD_TRAP_ID_ICMPV6_CONF_TYPE0,
    SX_TRAP_ID_USER_ICMPV6_CONF_TYPE1 = SXD_TRAP_ID_ICMPV6_CONF_TYPE1,
    SX_TRAP_ID_USER_OVERLAY_ICMPV6_CONF_TYPE = SXD_TRAP_ID_OVERLAY_ICMPV6_CONF_TYPE,
    SX_TRAP_ID_USER_NVE_DECAP_ETH = SXD_TRAP_ID_NVE_DECAP_ETH,

    SX_TRAP_ID_MIN = SX_TRAP_ID_GENERAL_FDB,          /**< MINIMUM TRAP ID */
    SX_TRAP_ID_MAX = SX_TRAP_ID_TRANSACTION_ERROR, /**< MAXIMUM TRAP ID */
} sx_trap_id_t;

#define SX_TRAP_ID_COUNT SX_TRAP_ID_MAX + 1
/**
 * SX_TRAP_ID_CHECK_RANGE defines Trap ID check range macro
 */
#define SX_TRAP_ID_CHECK_RANGE(TRAP_ID) \
    SX_CHECK_RANGE(SX_TRAP_ID_MIN, TRAP_ID, SX_TRAP_ID_MAX)

/**
 * SX_TRAP_ACL_CHECK_RANGE defines acl trap id check range macro
 */
#define SX_TRAP_ID_ACL_CHECK_RANGE(TRAP_ID) \
    SX_CHECK_RANGE(SX_TRAP_ID_ACL_MIN, TRAP_ID, SX_TRAP_ID_ACL_MAX)

/**
 * SX_TRAP_IPTRAP_CHECK_RANGE defines iptrap trap id check range macro
 */
#define SX_TRAP_ID_IPTRAP_CHECK_RANGE(TRAP_ID) \
    SX_CHECK_RANGE(SX_TRAP_ID_IPTRAP_MIN, TRAP_ID, SX_TRAP_ID_IPTRAP_MAX)

/**
 * SX_TRAP_SW_CHECK_RANGE defines iptrap trap id check range
 * macro
 */
#define SX_TRAP_ID_SW_CHECK_RANGE(TRAP_ID) \
    SX_CHECK_RANGE(SX_TRAP_ID_NEW_DEVICE_ADD, TRAP_ID, SX_TRAP_ID_FDB_IP_ADDR_ACTIVITY)

/**
 * SX_TRAP_SW_CHECK_RANGE defines iptrap trap id check range
 * macro
 */
#define SX_TRAP_ID_USER_L4_PORT_CHECK_RANGE(TRAP_ID) \
    SX_CHECK_RANGE(SX_TRAP_ID_USER_BASE, TRAP_ID, SX_TRAP_ID_USER_MAX)

static const uint32_t user_attr_traps_list[] = {
    SX_TRAP_ID_USER_ICMPV6_CONF_TYPE0,
    SX_TRAP_ID_USER_ICMPV6_CONF_TYPE1,
    SX_TRAP_ID_USER_OVERLAY_ICMPV6_CONF_TYPE,
    SX_TRAP_ID_USER_NVE_DECAP_ETH,
    SX_TRAP_ID_USER_ETH_CONF_TYPE0,
    SX_TRAP_ID_USER_ETH_CONF_TYPE1
};

#define SX_USER_ATTR_TRAPS_NUM (sizeof(user_attr_traps_list) / sizeof(user_attr_traps_list[0]))

#define SX_TRAP_ID_USER_ATTR_CHECK_RANGE(TRAP_ID)           \
    (TRAP_ID == SX_TRAP_ID_USER_ICMPV6_CONF_TYPE0 ||        \
     TRAP_ID == SX_TRAP_ID_USER_ICMPV6_CONF_TYPE1 ||        \
     TRAP_ID == SX_TRAP_ID_USER_OVERLAY_ICMPV6_CONF_TYPE || \
     TRAP_ID == SX_TRAP_ID_USER_NVE_DECAP_ETH ||            \
     TRAP_ID == SX_TRAP_ID_USER_ETH_CONF_TYPE0 ||           \
     TRAP_ID == SX_TRAP_ID_USER_ETH_CONF_TYPE1              \
    )

#define SX_TRAP_ID_USER_CHECK_RANGE(TRAP_ID)         \
    (SX_TRAP_ID_USER_L4_PORT_CHECK_RANGE(TRAP_ID) || \
     SX_TRAP_ID_USER_ATTR_CHECK_RANGE(TRAP_ID))

#define SX_USER_DEFINED_L4_PORTS_TRAPS_NUM (SX_TRAP_ID_USER_MAX - SX_TRAP_ID_USER_BASE + 1)
#define SX_USER_DEFINED_TRAPS_NUM          (SX_USER_DEFINED_L4_PORTS_TRAPS_NUM + SX_USER_ATTR_TRAPS_NUM)

/**
 * sx_trap_action_t enumerated type is used to note the supported trap actions
 */
typedef enum sx_trap_action {
    SX_TRAP_ACTION_IGNORE = SXD_TRAP_ACTION_IGNORE, /**< ignore action */
    SX_TRAP_ACTION_TRAP_2_CPU = SXD_TRAP_ACTION_TRAP_2_CPU, /**< trap to CPU action */
    SX_TRAP_ACTION_MIRROR_2_CPU = SXD_TRAP_ACTION_MIRROR_2_CPU, /**< Mirror to CPU action */
    SX_TRAP_ACTION_DISCARD = SXD_TRAP_ACTION_DISCARD, /**< Discard action */
    SX_TRAP_ACTION_SOFT_DISCARD = SXD_TRAP_ACTION_SOFT_DISCARD, /**< Soft discard action */
    SX_TRAP_ACTION_TRAP_SOFT_DISCARD = SXD_TRAP_ACTION_TRAP_SOFT_DISCARD, /**< Trap and soft discard action */
    SX_TRAP_ACTION_EXCEPTION_TRAP = SXD_TRAP_ACTION_EXCEPTION_TRAP, /**< Exception trap action */

    SX_TRAP_ACTION_MIN = SXD_TRAP_ACTION_MIN, /**< minimum action */
    SX_TRAP_ACTION_MAX = SXD_TRAP_ACTION_MAX /**< maximum action */
} sx_trap_action_t;

/**
 * SX_TRAP_ACTION_CHECK_RANGE defines trap action check range macro
 */
#define SX_TRAP_ACTION_CHECK_RANGE(TRAP_ACTION) \
    SX_CHECK_RANGE(SX_TRAP_ACTION_MIN, TRAP_ACTION, SX_TRAP_ACTION_MAX)

/**< Best effort priority */
#define SX_TRAP_PRIORITY_BEST_EFFORT 0
/**< Low priority */
#define SX_TRAP_PRIORITY_LOW 1
/**< Medium priority */
#define SX_TRAP_PRIORITY_MED 2
/**< High priority */
#define SX_TRAP_PRIORITY_HIGH 3
/**< Critical priority - EMADs/MADs only */
#define SX_TRAP_PRIORITY_CRITICAL 4
/**< Minimum priority */
#define SX_TRAP_PRIORITY_MIN SX_TRAP_PRIORITY_BEST_EFFORT
/**< Maximum priority since critical should not be configured by the user */
#define SX_TRAP_PRIORITY_MAX SX_TRAP_PRIORITY_HIGH

typedef uint32_t sx_trap_priority_t;

static __attribute__((__used__)) const char* sx_trap_priority_str[] = {
    "B/E",
    "LOW",
    "MED",
    "HIGH",
    "CRIT",
};

#define SX_TRAP_PRIORITY_STR_LEN (sizeof(sx_trap_priority_str) / sizeof(char*))

#define SX_TRAP_PRIORITY_STR(index)                      \
    (SX_CHECK_MAX(index, SX_TRAP_PRIORITY_STR_LEN - 1) ? \
     sx_trap_priority_str[index] : "UNKNOWN")

/**
 * SX_TRAP_PRIORITY_CHECK_RANGE defines trap priority check range macro,
 * for SwitchX and SwitchX-2 devices.
 */
#define SX_TRAP_PRIORITY_CHECK_RANGE(TRAP_PRIORITY) \
    SX_CHECK_MAX(TRAP_PRIORITY, SX_TRAP_PRIORITY_MAX)

/**
 * SX_TRAP_SPECTRUM_PRIORITY_CHECK_RANGE defines trap priority check range
 * macro, for SPECTRUM devices.
 */
#define SX_TRAP_SPECTRUM_PRIORITY_CHECK_RANGE(TRAP_PRIORITY) \
    SX_CHECK_MAX(TRAP_PRIORITY, SX_TRAP_SPECTRUM_PRIORITY_MAX)

/************************************************
 *  DEFINITIONS
 ***********************************************/

/**
 * sx_stclass_t is used to store stacking traffic class value
 */
typedef uint8_t sx_stclass_t;

/**
 * sx_rdq_t is used to store receive queue value
 */
typedef uint8_t sx_rdq_t;

/**
 * sx_packet_type_t enumerated type is used to note the type of the tx/rx packet
 */
typedef enum ku_pkt_type sx_packet_type_t;

/**
 * SX_PACKET_TYPE_CHECK_RANGE defines a packet type value check range macro
 */
#define SX_PACKET_TYPE_CHECK_RANGE(PACKET_TYPE) SX_CHECK_RANGE(SX_PKT_TYPE_MIN, (int)PACKET_TYPE, SX_PKT_TYPE_MAX)

/**
 * sx_packet_type_str is used to store packet types names
 */
static const char *sx_packet_type_str[] = {
    /*SX_PKT_TYPE_ETH_CTL_UC */
    "ETH packet (control - UC)",
    /*SX_PKT_TYPE_ETH_CTL_MC */
    "ETH packet (control - MC)",
    /*SX_PKT_TYPE_ETH_DATA */
    "ETH packet (data)",
    /*SX_PKT_TYPE_DROUTE_EMAD_CTL */
    "EMAD - direct route",
    /*SX_PKT_TYPE_EMAD_CTL */
    "EMAD - non direct route",
    /*SX_PKT_TYPE_FC_CTL_UC */
    "FC (control - UC)",
    /*SX_PKT_TYPE_FC_CTL_MC */
    "FC (control - MC)",
    /*SX_PKT_TYPE_FCOE_CTL_UC */
    "FCOE (control - UC)",
    /*SX_PKT_TYPE_FCOE_CTL_MC */
    "FCOE (control - MC)",
    /*SX_PKT_TYPE_IB_RAW_CTL */
    "IB (control - RAW)",
    /*SX_PKT_TYPE_IB_TRANSPORT_CTL */
    "IB (control - transport)",
    /*SX_PKT_TYPE_IB_RAW_DATA */
    "IB (data - RAW)",
    /*SX_PKT_TYPE_IB_TRANSPORT_DATA */
    "IB (data - transport)",
    /*SX_PKT_TYPE_EOIB_CTL */
    "ETH OVER IB (control)",
    /*SX_PKT_TYPE_FCOIB_CTL */
    "FC OVER IB (control)",
    /* SX_PKT_TYPE_LOOPBACK_CTL */
    "SX_PKT_TYPE_LOOPBACK_CTL"
};

/**
 * sx_packet_type_str_len is used to store packet types names array length
 */
static const int sx_packet_type_str_len = sizeof(sx_packet_type_str) / sizeof(char*);

/**
 * SX_PACKET_TYPE_STR defines a packet type value to string macro
 */
#define SX_PACKET_TYPE_STR(PACKET_TYPE)                                          \
    (SX_PACKET_TYPE_CHECK_RANGE(PACKET_TYPE) ? sx_packet_type_str[PACKET_TYPE] : \
     "UNKNOWN")

/* max event buffer size for events about 10K */

#define SX_HOST_EVENT_BUFFER_SIZE_MAX 10500

/**
 * SX_ROUTER_ACTIVITY_NOTIFY_CNT_MAX determines the maximal number of entries
 * which can be received in the router activity notify event
 * */
#define SX_ROUTER_ACTIVITY_NOTIFY_CNT_MAX     128
#define SX_FDB_IGMPV3_ACTIVITY_NOTIFY_CNT_MAX 128
/************************************************
 *  MACROS
 ***********************************************/

/**
 * SX_API_GET_RELEVANT_PACKET_TYPE defines a trap id / event id to syndrome id macro
 */
#define SX_API_GET_RELEVANT_PACKET_TYPE(EVENT_ID, TYPE)                                                  \
    {                                                                                                    \
        if (EVENT_ID == SX_TRAP_ID_FCOE_FIP) {                                                           \
            TYPE = SX_KU_L2_TYPE_FC;                                                                     \
        }                                                                                                \
        else if (EVENT_ID == SX_TRAP_ID_QUEUE_THRESHOLD_CROSSED) {                                       \
            TYPE = SX_KU_L2_TYPE_DONT_CARE;                                                              \
        }                                                                                                \
        else if (EVENT_ID == SX_TRAP_ID_ETH_L2_PACKET_SAMPLING) {                                        \
            TYPE = SX_KU_L2_TYPE_DONT_CARE;                                                              \
        }                                                                                                \
        else if (SX_CHECK_RANGE(SX_TRAP_ID_ETH_L2_STP, EVENT_ID, SX_TRAP_ID_ETH_L2_RARP_OPCODES)) {      \
            TYPE = SX_KU_L2_TYPE_ETH;                                                                    \
        }                                                                                                \
        else if (SX_CHECK_RANGE(SX_TRAP_ID_INFINIBAND_QP0, EVENT_ID, SX_TRAP_ID_INFINIBAND_OTHER_QPS)) { \
            TYPE = SX_KU_L2_TYPE_IB;                                                                     \
        }                                                                                                \
        else if (EVENT_ID == SX_TRAP_ID_ARP_REQUEST) {                                                   \
            TYPE = SX_KU_L2_TYPE_ETH;                                                                    \
        }                                                                                                \
        else if (EVENT_ID == SX_TRAP_ID_ARP_RESPONSE) {                                                  \
            TYPE = SX_KU_L2_TYPE_ETH;                                                                    \
        }                                                                                                \
        else if (SX_CHECK_RANGE(SX_TRAP_ID_ACL_MIN, EVENT_ID, SX_TRAP_ID_ACL_MAX)) {                     \
            TYPE = SX_KU_L2_TYPE_ETH;                                                                    \
        }                                                                                                \
        else {                                                                                           \
            TYPE = SX_KU_L2_TYPE_DONT_CARE;                                                              \
        }                                                                                                \
    }

/**
 * SX_API_GET_DROP_ENABLED defines whether a trap id / event id can drops
 * events in driver
 */
#define SX_API_GET_DROP_ENABLED(EVENT_ID, DROP_E)                  \
    {                                                              \
        if (EVENT_ID == SX_TRAP_ID_BER_MONITOR) {                  \
            DROP_E = 1;                                            \
        }                                                          \
        else if (EVENT_ID == SX_TRAP_ID_QUEUE_THRESHOLD_CROSSED) { \
            DROP_E = 1;                                            \
        }                                                          \
        else {                                                     \
            DROP_E = 0;                                            \
        }                                                          \
    }

#endif /* SX_TRAP_ID_H_ */
