/*
 * Copyright (C) Mellanox Technologies, Ltd. 2010-2018 ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of Mellanox Technologies, Ltd.
 * (the "Company") and all right, title, and interest in and to the software product,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __SXD_EMAD_LAG_H__
#define __SXD_EMAD_LAG_H__

#include <complib/sx_log.h>

#include <sx/sxd/sxd_types.h>
#include <sx/sxd/sxd_emad_lag_data.h>

/************************************************
 *  Local Defines
 ***********************************************/

/************************************************
 *  Local Macros
 ***********************************************/

/************************************************
 *  Local Type definitions
 ***********************************************/

/************************************************
 *  Defines
 ***********************************************/

/************************************************
 *  Macros
 ***********************************************/

/************************************************
 *  Type definitions
 ***********************************************/

/************************************************
 *  Global variables
 ***********************************************/

/************************************************
 *  Function declarations
 ***********************************************/

/************************************************
 *  API functions
 ***********************************************/

/**
 * This function sets the log verbosity level of EMAD LAG MODULE
 * @param[in] cmd               - SET / GET
 * @param[in] verbosity_level_p - verbosity level
 *
 * @return SX_STATUS_SUCCESS operation completes successfully
 *          SX_STATUS_ERROR   general error
 */
sxd_status_t sxd_emad_lag_log_verbosity_level(IN sxd_access_cmd_t      cmd,
                                              IN sx_verbosity_level_t *verbosity_level_p);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 *  This function sets  SFDT Switch LAG Descriptor fields=
 * @param[in]  sfdt_data_arr SFDT EMAD data array pointer
 * @param[in]  sfdt_data_num number of SFDT EMAD data
 * @param[in]  handler  sender ID
 * @param[in,out]  context
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_sfdt_set(sxd_emad_sfdt_data_t         *sfdt_data_arr,
                               uint32_t                      sfdt_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);


/**
 *  This function gets  SFDT Switch LAG Descriptor fields=
 * @param[in,out]  sfdt_data_arr SFDT EMAD data array pointer
 * @param[in]  sfdt_data_num number of SFDT EMAD data
 * @param[in]  handler  sender ID
 * @param[in,out]  context
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_sfdt_get(sxd_emad_sfdt_data_t         *sfdt_data_arr,
                               uint32_t                      sfdt_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 *  This function sets  SLDR Switch LAG Descriptor fields=
 * @param[in]  sldr_data_arr SLDR EMAD data array pointer
 * @param[in]  sldr_data_num number of SLDR EMAD data
 * @param[in]  handler  sender ID
 * @param[in,out]  context
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_sldr_set(sxd_emad_sldr_data_t         *sldr_data_arr,
                               uint32_t                      sldr_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);


/**
 *  This function gets  SLDR Switch LAG Descriptor fields=
 * @param[in,out]  sldr_data_arr SLDR EMAD data array pointer
 * @param[in]  sldr_data_num number of SLDR EMAD data
 * @param[in]  handler  sender ID
 * @param[in,out]  context
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_sldr_get(sxd_emad_sldr_data_t         *sldr_data_arr,
                               uint32_t                      sldr_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function sets  SLCR Switch LAG Configuration Register
 *  -sets the LAG hash parameter
 *
 * @param[in]  slcr_data_arr SLCR EMAD data array pointer
 * @param[in]  slcr_data_num number of SLCR EMAD data
 * @param[in]  handler  sender ID
 * @param[in,out]  context
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_slcr_set(sxd_emad_slcr_data_t         *slcr_data_arr,
                               uint32_t                      slcr_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function gets  SLCR Switch LAG Configuration Register
 *  -gets the LAG hash parameter
 *
 * @param[in,out]  slcr_data_arr SLCR EMAD data array pointer
 * @param[in]  slcr_data_num number of SLCR EMAD data
 * @param[in]  handler  sender ID
 * @param[in,out]  context
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_slcr_get(sxd_emad_slcr_data_t         *slcr_data_arr,
                               uint32_t                      slcr_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function gets  SLCOR - Switch LAG Collector Register
 *
 * @param[in,out]  slcor_data_arr SLCOR EMAD data array pointer
 * @param[in]  slcor_data_num number of SLCOR EMAD data
 * @param[in]  handler  sender ID
 * @param[in,out]  context
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_slcor_get(sxd_emad_slcor_data_t        *slcor_data_arr,
                                uint32_t                      slcor_data_num,
                                sxd_emad_completion_handler_t handler,
                                void                         *context);
/**
 *  This function sets SLCOR Switch LAG Collector Register
 *
 * @param[in]  slcor_data_arr SLCOR EMAD data array pointer
 * @param[in]  slcor_data_num number of SLCOR EMAD data
 * @param[in]  handler  sender ID
 * @param[in,out]  context
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_slcor_set(sxd_emad_slcor_data_t        *slcor_data_arr,
                                uint32_t                      slcor_data_num,
                                sxd_emad_completion_handler_t handler,
                                void                         *context);

#endif /* __SXD_EMAD_LAG_H__ */
