/*
 * Copyright (C) Mellanox Technologies, Ltd. 2010-2018 ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of Mellanox Technologies, Ltd.
 * (the "Company") and all right, title, and interest in and to the software product,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __SXD_EMAD_REDECN_REG_H__
#define __SXD_EMAD_REDECN_REG_H__

#include <sx/sxd/sxd_types.h>

/************************************************
 *  Local Defines
 ***********************************************/

/************************************************
 *  Local Macros
 ***********************************************/

/************************************************
 *  Local Type definitions
 ***********************************************/

/************************************************
 *  Defines
 ***********************************************/

/************************************************
 *  Macros
 ***********************************************/

/************************************************
 *  Type definitions
 ***********************************************/

#include <complib/cl_packon.h>

/**
 * sxd_emad_cwgcr_reg structure is used to store cwgcr register
 * layout.
 */
typedef struct sxd_emad_cwgcr_reg {
    uint8_t en;
    uint8_t scd;
    uint8_t aqs_weight;
    uint8_t aqs_time;
    uint8_t reserved[3];
    uint8_t cece;
} PACK_SUFFIX sxd_emad_cwgcr_reg_t;

/**
 * sxd_emad_cwtp_reg structure is used to store cwtp register
 * layout.
 */
typedef struct sxd_emad_cwtp_reg {
    uint8_t  reserved0;
    uint8_t  local_port;
    uint16_t reserved4;
    uint8_t  reserved1[31];
    uint8_t  traffic_class;
    uint8_t  mode;
    uint8_t  reserved2[3];
    uint32_t profile1_min;
    uint32_t profile1_percent_profile1_max;
    uint32_t profile2_min;
    uint32_t profile2_percent_profile2_max;
    uint32_t profile3_min;
    uint32_t profile3_percent_profile3_max;
} PACK_SUFFIX sxd_emad_cwtp_reg_t;


/**
 * sxd_emad_qprt_reg structure is used to store cwtpm register
 * layout.
 */
typedef struct sxd_emad_cwtpm_reg {
    uint8_t  reserved0;
    uint8_t  local_port;
    uint16_t reserved00;
    uint8_t  reserved1[31];
    uint8_t  traffic_class;
    uint8_t  reserved2[3];
    uint8_t  ew_ee;
    uint8_t  reserved3[12];
    uint8_t  reserved4[3];
    uint8_t  tcp_g;
    uint8_t  reserved5;
    uint8_t  tcp_y;
    uint8_t  reserved6;
    uint8_t  tcp_r;
    uint8_t  reserved7[3];
    uint8_t  ntcp_g;
    uint8_t  reserved8;
    uint8_t  ntcp_y;
    uint8_t  reserved9;
    uint8_t  ntcp_r;
} PACK_SUFFIX sxd_emad_cwtpm_reg_t;

/**
 * sxd_emad_qprt_reg structure is used to store cwpp register
 * layout.
 */
typedef struct sxd_emad_cwpp_reg {
    uint8_t reserved1[3];
    uint8_t pool;
    uint8_t reserved2[4];
    uint8_t reserved3[1];
    uint8_t profile1_min[3];
    uint8_t profile1_percent;
    uint8_t profile1_max[3];
    uint8_t reserved4[1];
    uint8_t profile2_min[3];
    uint8_t profile2_percent;
    uint8_t profile2_max[3];
    uint8_t reserved5[1];
    uint8_t profile3_min[3];
    uint8_t profile3_percent;
    uint8_t profile3_max[3];
} PACK_SUFFIX sxd_emad_cwpp_reg_t;

/**
 * sxd_emad_qprt_reg structure is used to store cwppm register
 * layout.
 */
typedef struct sxd_emad_cwppm_reg {
    uint8_t reserved1[3];
    uint8_t pool;
    uint8_t reserved2[3];
    uint8_t tcp_g;
    uint8_t reserved3;
    uint8_t tcp_y;
    uint8_t reserved4;
    uint8_t tcp_r;
    uint8_t reserved5[3];
    uint8_t ntcp_g;
    uint8_t reserved6;
    uint8_t ntcp_y;
    uint8_t reserved7;
    uint8_t ntcp_r;
} PACK_SUFFIX sxd_emad_cwppm_reg_t;

/**
 * sxd_emad_qprt_reg structure is used to store cpqe register
 * layout.
 */
typedef struct sxd_emad_cpqe_reg {
    uint8_t  reserved1;
    uint8_t  local_port;
    uint8_t  reserved2[2];
    uint32_t reserved3;
    uint8_t  reserved5;
    uint8_t  marking_percent;
    uint8_t  reserved6[2];
} PACK_SUFFIX sxd_emad_cpqe_reg_t;


#include <complib/cl_packoff.h>

/************************************************
 *  Global variables
 ***********************************************/

/************************************************
 *  Function declarations
 ***********************************************/

#endif /* __SXD_EMAD_REDECN_REG_H__ */
