/*
 * Copyright (C) Mellanox Technologies, Ltd. 2010-2018 ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of Mellanox Technologies, Ltd.
 * (the "Company") and all right, title, and interest in and to the software product,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __SXD_EMAD_VLAN_H__
#define __SXD_EMAD_VLAN_H__

#include <complib/sx_log.h>

#include <sx/sxd/sxd_types.h>
#include <sx/sxd/sxd_emad_vlan_data.h>

/************************************************
 *  Local Defines
 ***********************************************/

/************************************************
 *  Local Macros
 ***********************************************/

/************************************************
 *  Local Type definitions
 ***********************************************/

/************************************************
 *  Macros
 ***********************************************/

/************************************************
 *  Type definitions
 ***********************************************/

/************************************************
 *  Global variables
 ***********************************************/

/************************************************
 *  Function declarations
 ***********************************************/

/**
 * This function sets the log verbosity level of EMAD VLAN MODULE
 * @param[in] cmd               - SET / GET
 * @param[in] verbosity_level_p - verbosity level
 *
 * @return SX_STATUS_SUCCESS operation completes successfully
 *          SX_STATUS_ERROR   general error
 */
sxd_status_t sxd_emad_vlan_log_verbosity_level(IN sxd_access_cmd_t      cmd,
                                               IN sx_verbosity_level_t *verbosity_level_p);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spvid_set(sxd_emad_spvid_data_t        *spvid_data_arr,
                                uint32_t                      spvid_data_num,
                                sxd_emad_completion_handler_t handler,
                                void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spvid_get(sxd_emad_spvid_data_t        *spvid_data_arr,
                                uint32_t                      spvid_data_num,
                                sxd_emad_completion_handler_t handler,
                                void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spvm_set(sxd_emad_spvm_data_t         *spvm_data_arr,
                               uint32_t                      spvm_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spvm_get(sxd_emad_spvm_data_t         *spvm_data_arr,
                               uint32_t                      spvm_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spvtr_set(sxd_emad_spvtr_data_t        *spvtr_data_arr,
                                uint32_t                      spvtr_data_num,
                                sxd_emad_completion_handler_t handler,
                                void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spvtr_get(sxd_emad_spvtr_data_t        *spvtr_data_arr,
                                uint32_t                      spvtr_data_num,
                                sxd_emad_completion_handler_t handler,
                                void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spaft_set(sxd_emad_spaft_data_t        *spaft_data_arr,
                                uint32_t                      spaft_data_num,
                                sxd_emad_completion_handler_t handler,
                                void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spaft_get(sxd_emad_spaft_data_t        *spaft_data_arr,
                                uint32_t                      spaft_data_num,
                                sxd_emad_completion_handler_t handler,
                                void                         *context);

sxd_status_t sxd_emad_spvc_set(sxd_emad_spvc_data_t         *spvc_data_arr,
                               uint32_t                      spvc_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_spvc_get(sxd_emad_spvc_data_t         *spvc_data_arr,
                               uint32_t                      spvc_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_sver_set(sxd_emad_sver_data_t         *sver_data_arr,
                               uint32_t                      sver_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);

/**
 *  This function closes channel to SX-API operations.
 *
 * @param[in] handle - SX-API handle.
 *
 * @return sxd_status_t
 */
sxd_status_t sxd_emad_sver_get(sxd_emad_sver_data_t         *sver_data_arr,
                               uint32_t                      sver_data_num,
                               sxd_emad_completion_handler_t handler,
                               void                         *context);


#endif /* __SXD_EMAD_VLAN_H__ */
