/*
 * Copyright (C) Mellanox Technologies, Ltd. 2010-2018 ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of Mellanox Technologies, Ltd.
 * (the "Company") and all right, title, and interest in and to the software product,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __SXD_PORT_H__
#define __SXD_PORT_H__

#include <complib/cl_types.h>
#include <sx/sxd/sxd_check.h>

/************************************************
 *  Local Defines
 ***********************************************/

/************************************************
 *  Local Macros
 ***********************************************/

/************************************************
 *  Local Type definitions
 ***********************************************/

/************************************************
 *  Macros
 ***********************************************/

/************************************************
 *  Type definitions
 ***********************************************/

/************************************************
 *  Global variables
 ***********************************************/

/************************************************
 *  Function declarations
 ***********************************************/


#define SXD_PORT_SYS_ID_MIN             (0x00000000)
#define SXD_PORT_SYS_ID_MAX             (0x0000FFFF)
#define SXD_PORT_PHY_ID_MIN             (0x00000000)
#define SXD_PORT_PHY_ID_MAX             (0x000000FF)
#define SXD_PORT_SUB_ID_MIN             (0x00000000)
#define SXD_PORT_SUB_ID_MAX             (0x0000000F)
#define SXD_PORT_MAX_NUM                (64)
#define SXD_PORT_TUNING_OVERRIDE_ENABLE (1)

#define SXD_PORT_SYS_ID_CHECK_RANGE(PORT_SYS_ID) SXD_CHECK_RANGE(SXD_PORT_SYS_ID_MIN, PORT_SYS_ID, SXD_PORT_SYS_ID_MAX)
#define SXD_PORT_PHY_ID_CHECK_RANGE(PORT_PHY_ID) SXD_CHECK_RANGE(SXD_PORT_PHY_ID_MIN, PORT_PHY_ID, SXD_PORT_PHY_ID_MAX)
#define SXD_PORT_SUB_ID_CHECK_RANGE(PORT_SUB_ID) SXD_CHECK_RANGE(SXD_PORT_SUB_ID_MIN, PORT_SUB_ID, SXD_PORT_SUB_ID_MAX)

/* ******************************* */
/* EMAD port definitions          */
/* ******************************* */
#define SXD_PORT_PBMC_NUM_BUFF                                10
#define SXD_PORT_PPCNT_MAX_CNT_NUM                            31
#define SXD_PORT_PPCNT_CNTR_GRP_IB_PORT_CNTRS                 0x20
#define SXD_PORT_PPCNT_CNTR_GRP_IB_PORT_CNTRS_EXTENDED        0x21
#define SXD_PORT_PPCNT_CNTR_GRP_IB_PORT_RCV_ERR               0x22
#define SXD_PORT_PPCNT_CNTR_GRP_IB_PORT_XMIT_DISCARD          0x23
#define SXD_PORT_PPCNT_CNTR_GRP_IB_PORT_FLOW_CTL_CNTRS        0x24
#define SXD_PORT_PPCNT_CNTR_GRP_IB_PORT_VL_XMIT_WAIT_CNTRS    0x25
#define SXD_PORT_PPCNT_CNTR_GRP_IB_PORT_SW_PORT_VL_CONGESTION 0x26


/* Port System ID:
 *****************
 * Physical entity, that represents the HW's a destination port in the GLOBAL system:
 ************************************************************************************
 * +--------------------------------------------------------------------------------+
 * |                                 Port-Sys-ID[15:0]                              |
 * +--------------------------------------------------------------------------------+
 ************************************************************************************
 */
typedef uint16_t sxd_port_sys_id_t;

/* Port Physical ID:
 *******************
 * Physical entity, that represents the port's actual ID in the LOCAL (device) system
 ************************************************************************************
 * +--------------------------------------------------------------------------------+
 * |                                 Port-Phy-ID[7:0]                               |
 * +--------------------------------------------------------------------------------+
 ************************************************************************************
 */
typedef uint8_t sxd_port_phy_id_t;
typedef sxd_port_phy_id_t sxd_port_mod_id_t;

/* Port Sub ID:
 **************
 * Logical entity that represents the format of a (VEPA) channel in the PHYSICAL port
 ************************************************************************************
 * +--------------------------------------------------------------------------------+
 * |                                 Port-Sub-ID[7:0]                               |
 * +--------------------------------------------------------------------------------+
 ************************************************************************************
 */
typedef uint8_t sxd_port_sub_id_t;

/**
 * sxd_port_status_t enumerated type is used to store port
 * status.
 */
typedef enum sxd_port_status {
    SXD_PORT_STATUS_UP = (1 << 0),
    SXD_PORT_STATUS_DOWN = (1 << 1),
    SXD_PORT_STATUS_FAILURE = (1 << 2),
} sxd_port_status_t;

/**
 * sxd_port_admin_status_t enumerated type is used to store port
 * administrative status.
 */
typedef enum sxd_port_admin_status {
    SXD_PORT_ADMIN_STATUS_UP = 0x1,
    SXD_PORT_ADMIN_STATUS_DOWN_BY_CONF = 0x2,
    SXD_PORT_ADMIN_STATUS_UP_ONCE = 0x3,
} sxd_port_admin_status_t;

/**
 * sxd_port_oper_status_t enumerated type is used to store port
 * operational status.
 */
typedef enum sxd_port_oper_status {
    SXD_PORT_OPER_STATUS_UP = (1 << 0),
    SXD_PORT_OPER_STATUS_DOWN = (1 << 1),
    SXD_PORT_OPER_STATUS_DOWN_BY_FAIL = (1 << 2),
} sxd_port_oper_status_t;

/**
 * sxd_port_module_status_t enumerated type is used to store port
 * module status.
 */
typedef enum sxd_port_module_status {
    SXD_PORT_MODULE_STATUS_PLUGGED = 0x1,
    SXD_PORT_MODULE_STATUS_UNPLUGGED = 0x2,
    SXD_PORT_MODULE_STATUS_PLUGGED_WITH_ERROR = 0x3,
    SXD_PORT_MODULE_STATUS_PLUGGED_DISABLED = 0x4,
} sxd_port_module_status_t;

/**
 *  * sxd_port_module_error_type_t enumerated type is used to store port
 *   * module error types.
 *    */
typedef enum sxd_port_module_error_type {
    SXD_PORT_MODULE_ERROR_POWER_BUDGET_EXCEEDED = 0x0,
    SXD_PORT_MODULE_ERROR_LONG_RANGE_FOR_NON_MLNX_MODULE = 0x1,
    SXD_PORT_MODULE_ERROR_BUS_STUCK = 0x2,
    SXD_PORT_MODULE_ERROR_UNSUPPORTED_EEPROM = 0x3,
    SXD_PORT_MODULE_ERROR_ENFORCE_PART_NUMBER_LIST = 0x4,
    SXD_PORT_MODULE_ERROR_UNSUPPORTED_CABLE = 0x5,
    SXD_PORT_MODULE_ERROR_HIGH_TEMP = 0x6,
    SXD_PORT_MODULE_ERROR_BAD_CABLE = 0x7,
} sxd_port_module_error_type_t;

/**
 * sxd_port_event_generate_mode_t enumerated type is used to store port
 * generate event mode.
 */
typedef enum sxd_port_event_generate_mode {
    SXD_PORT_EVENT_GENERATE_MODE_DONT = 0,
    SXD_PORT_EVENT_GENERATE_MODE_GENERATE = 1,
    SXD_PORT_EVENT_GENERATE_MODE_GENERATE_SINGLE = 2
} sxd_port_event_generate_mode_t;

#define SX_PORT_CAPABILITIES_32B_MASK 4

/**
 * sxd_port_prio_id_buff_t enumerated type is used to store
 * buffer priority id
 */
typedef enum sxd_port_prio_id_buff {
    SXD_PORT_PRIORITY_0_BUFFER = (0),
    SXD_PORT_PRIORITY_1_BUFFER = (1),
    SXD_PORT_PRIORITY_2_BUFFER = (2),
    SXD_PORT_PRIORITY_3_BUFFER = (3),
    SXD_PORT_PRIORITY_4_BUFFER = (4),
    SXD_PORT_PRIORITY_5_BUFFER = (5),
    SXD_PORT_PRIORITY_6_BUFFER = (6),
    SXD_PORT_PRIORITY_7_BUFFER = (7),
    SXD_PORT_PRIORITY_8_BUFFER = (8),
    SXD_PORT_PRIORITY_9_BUFFER = (9),
} sxd_port_prio_id_buff_t;

/**
 * sxd_mask_t enumerated type is used to store mask
 */
typedef enum sxd_mask {
    SXD_VALID,
    SXD_INVALID,
} sxd_mask_t;

/**
 * sxd_port_policer_bind_operation_t enumerated type is used to store port
 * policer bind operation.
 */
typedef enum sxd_port_policer_bind_operation {
    SXD_PORT_POLICER_BIND = 0,
    SXD_PORT_POLICER_UPDATE = 1,
    SXD_PORT_POLICER_UNBIND = 2,
} sxd_port_policer_bind_operation_t;

/**
 * sxd_port_ib_proto_speed_bit_t enumerated type is used to store the infiniband port
 * speed bit offset for each possible speed, as configured in PTYS register
 */
typedef enum sxd_port_ib_proto_speed_bit {
    SXD_PORT_IB_PROTO_SPEED_SDR = 0,
    SXD_PORT_IB_PROTO_SPEED_DDR = 1,
    SXD_PORT_IB_PROTO_SPEED_QDR = 2,
    SXD_PORT_IB_PROTO_SPEED_FDR10 = 3,
    SXD_PORT_IB_PROTO_SPEED_FDR = 4,
    SXD_PORT_IB_PROTO_SPEED_EDR = 5,
} sxd_port_ib_proto_speed_bit_t;

/**
 * sxd_port_ib_proto_width_bit_t enumerated type is used to store the infiniband port
 * width bit offset for each possible width as configured in PTYS register
 */
typedef enum sxd_port_ib_proto_width_bit {
    SXD_PORT_IB_PROTO_WIDTH_1X = 0,
    SXD_PORT_IB_PROTO_WIDTH_4X = 2,
} sxd_port_ib_proto_width_bit_t;

/**
 * sxd_port_profile_mode_t enumerated type is used to store port
 * profile mode.
 */
typedef enum sxd_port_profile_mode {
    SXD_PORT_TYPE_BASED_PROFILE = (1 << 0),
    SXD_PORT_LEGACY_IBSPEC_PROFILE = (1 << 1),
} sxd_port_profile_mode_t;


/**
 * sxd_port_fec_mode_t enumerated type is used to store port
 * FEC mode.
 */
typedef enum sxd_port_fec_mode {
    SXD_PORT_NO_FEC = (1 << 0),
    SXD_PORT_FIRECODE_FEC = (1 << 1),
    SXD_PORT_RS_FEC = (1 << 2),
    SXD_PORT_LL_RS_FEC = (1 << 3),
} sxd_port_fec_mode_t;

#endif /* __SXD_PORT_H__ */
